#if !defined(_GLACIERLINEHISTORY_H_)
#define _GLACIERLINEHISTORY_H_

#include <stack>
#include "glimslinedata.h"
#include "TraceLog.h"

/**
 * Actions a user can perform on a line or node.
 * GlacierLineHistoryElements are tagged with
 * the action the user performed on the Shape that
 * caused the element to be added to the history.
 * This information is used to take the
 * appropriate undo action, such as restoring
 * a deleted Node.
 *
 * @author deborah lee soltesz
 * @date   2005-07-06
 */
enum HistoryAction {
  noAction,                  //!< no action was taken
  deletedNode, deletedLine,  //!< a line or node was deleted
  addedNode, addedLine,      //!< a line or node was added
  insertedNode,              //!< a node (line vertex) was inserted into a line
  movedNode,                 //!< a node (line vertex) was moved
  joinedLines,               //!< two lines were joined into one
  brokeLine,                 //!< one line was broken into two lines
  selection                  //!< Selection actions are stored on their own stack in GLIMSLineData
} ;

/**
 * A single user action on a GlacierLine. The
 * modified Shape (GlacierLine or a Node it contains),
 * location indices in the original data structures,
 * and type of action taken are stored so a
 * deleted or otherwise modified Shape can be
 * restored to its previous state.
 *
 * @author deborah lee soltesz
 * @date   2005-07-06
 */
class GlacierLineHistoryElement {

private:
  Shape *       mGlacierLineComponent ; //!< Pointer to a GlacierLine or Node.
                                        //!< Used to hold a copy of a Shape object so it can be restored later.
  int           mLineIndex ;            //!< Location in the GLIMSLineData stack of the line/node that was changed.
  int           mNodeIndex ;            //!< Location in the GlacierLine stack where the changed node was located.
                                        //!< If the shape is a line or the action was a selection, the value should be -1.
  HistoryAction mAction ;               //!< Action taken by user on this shape.

public:
  /**
   * Default constructor for an empty history element
   *
   */
  GlacierLineHistoryElement() {
    mGlacierLineComponent = NULL ;
    mLineIndex = -1 ;
    mNodeIndex = -1 ;
    mAction = noAction ;
  }

  /**
   * Constructor a new history element with information to be stored
   * for a GlacierLine
   *
   * @param shp    The shape (GlacierLine) modified by user action for which the class will store history information
   * @param iline  The line's original position in the line set before modificaiton
   * @param inode  The position of a node in the line (-1 for "not applicable)
   * @param action The action taken on this line by the user to cause a history element to be stored
   *
   */
  GlacierLineHistoryElement(GlacierLine & shp, int iline, int inode, HistoryAction action) {

    GlacierLine * newLine = new GlacierLine ;
    (*newLine) = shp ;
    mGlacierLineComponent = newLine ;

    mLineIndex = iline ;
    mNodeIndex = inode ;
    mAction    = action ;

  }

  /**
   * Constructor a new history element with information to be stored
   * for a Node in a line
   *
   * @param shp    The shape (Node) modified by user action for which the class will store history information
   * @param iline  The line (referenced by index) the node belonged to before modificaiton
   * @param inode  The position of the node in the line
   * @param action The action taken on this node by the user to cause a history element to be stored
   *
   */
  GlacierLineHistoryElement(Node & shp, int iline, int inode, HistoryAction action) {
    Node * newNode = new Node ;
    (*newNode) = shp ;
    mGlacierLineComponent = newNode ;
    mLineIndex = iline ;
    mNodeIndex = inode ;
    mAction    = action ;
  }

  virtual ~GlacierLineHistoryElement() {}

  /**
   * Get/Set the action information from the history element
   */
  HistoryAction getAction    ()                        { return mAction ; }
  void          setAction    (HistoryAction newAction) { mAction = newAction ; }

  /**
   * Get/Set the shape  (GlacierLine or Node) from the history element
   */
  Shape *       getShape     ()                 { return mGlacierLineComponent ; }

  void          setShape     (GlacierLine * newShape) {
    GlacierLine * newLine = new GlacierLine ;
    (*newLine) = (*newShape) ;
    mGlacierLineComponent = newLine ;
  }

  void          setShape     (Node * newShape) {
    Node * newNode = new Node ;
    (*newNode) = (*newShape) ;
    mGlacierLineComponent = newNode ;
  }

  /**
   * Get/Set the index to the line where the shape was stored from the history element
   */
  int           getLineIndex ()       { return mLineIndex ; }
  void          setLineIndex (int li) { mLineIndex = li; }

  /**
   * Get/Set the index of the Node in the line where the shape was stored from the history element.
   * If the shape is a line or the action was a selection, this will return -1.
   */
  int           getNodeIndex ()       { return mNodeIndex ; }
  void          setNodeIndex (int ni) { mNodeIndex = ni ; }

} ;

/**
 * A container for holding and accessing the history of
 * user actions (GlacierLineHistoryElements) on a GlacierLine.
 *
 * @author deborah lee soltesz
 * @date   2005-07-06
 *
 * @history 2005-07-15 deborah lee soltesz
 *                     push (GlacierLineHistoryElement * historyElement)
 *                     was added to allow creation of a redo stack
 *                     from GlacierLineHistoryElements stored on the
 *                     history (undo) stack.
 *
 * @todo 2005-07-12 dls Tweak design to create abstract undo class,
 *                      and reorganize as is appropriate, removing
 *                      old undo structures (mUndoLL and mUndoXY).
 */
class GlacierLineHistory {
private:
  //! A stack (last in, first out) of history elements.
  stack <GlacierLineHistoryElement*> mHistory  ;

public:
  GlacierLineHistory() {;}
  virtual ~GlacierLineHistory() {;}

  /**
   * Put a shape (GlacierLine) and information about the
   * action that occurred to the shape on
   * the history stack.
   *
   * @param shp    The shape (GlacierLine) modified by user action for which the class will store history information
   * @param iline  The line's original position in the line set before modificaiton
   * @param inode  The position of a node in the line (-1 for "not applicable)
   * @param action The action taken on this line by the user to cause a history element to be stored
   */
  void push (GlacierLine & shp, int iline, int inode, HistoryAction action) {
    GlacierLineHistoryElement * newElem = new GlacierLineHistoryElement (shp, iline, inode, action) ;
    mHistory.push (newElem) ;
  }

  /**
   * Put a shape (Node) and information about the
   * action that occurred to the shape on
   * the history stack.
   *
   * @param shp    The shape (Node) modified by user action for which the class will store history information
   * @param iline  The line (referenced by index) the node belonged to before modificaiton
   * @param inode  The position of the node in the line
   * @param action The action taken on this node by the user to cause a history element to be stored
   */
  void push (Node & shp, int iline, int inode, HistoryAction action) {
    GlacierLineHistoryElement * newElem = new GlacierLineHistoryElement (shp, iline, inode, action) ;
    mHistory.push (newElem) ;
  }

  /**
   * Put a history element (modified node or line and info about the
   * action that occurred to the shape) on the history stack.
   *
   * @param historyElement An existing GlacierLineHistoryElement
   *                       to add to the history stack
   */
  void push (GlacierLineHistoryElement * historyElement) {
    mHistory.push (historyElement) ;
  }

  /**
   * Take a shape and information about the
   * action that occurred to the shape off
   * the history stack
   *
   * @return The top element in the history stack
   */
  GlacierLineHistoryElement * pop () {
    if (mHistory.size()) {
      GlacierLineHistoryElement * element = mHistory.top () ;
      mHistory.pop () ;
      return element ;
    }
    else return NULL ;
  }

  /**
   * Return the last action -- a shape and information about the
   * action that occurred to the shape -- without popping it
   * off the stack
   *
   * @return The top element in the history stack
   */
  GlacierLineHistoryElement * peek () {
    if (mHistory.size()) {
      GlacierLineHistoryElement * element = mHistory.top () ;
      return element ;
    }
    else return NULL ;
  }

  /**
   * Take a shape and information about the
   * action that occurred to the shape off
   * the history stack
   *
   * @return The top element in the history stack
   */
  void clear () {
    for (unsigned int i = 0 ; i < mHistory.size() ; i++) {
      mHistory.pop() ;
    }
  }

  /**
   * How many history items are stored in the history?
   *
   * @return Number of elements in the history
   */
  int size() {
    return mHistory.size() ;
  }

};

#endif // !defined(_GLACIERLINEHISTORY_H_)



