#if !defined(_TRACELOG_H_)
#define _TRACELOG_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <strstream>
#include <time.h>
using namespace std;

/**
 * TraceLog is a simple debugging tool which allows the
 * programmer to write messages to glimsview_log.txt.
 * Messages are stored during runtime and written
 * to the file when the GLIMSView application exits.
 * Disabling this functionality is done simply by
 * commenting out the code in this class that opens
 * the file and stores messages for writing to the file.
 * The TraceLog object, glimslog, is instantiated
 * near main in the maintoolbar.cpp file. To use
 * this object, include the TraceLog.h file, declare
 * extern TraceLog glimslog in the function that will
 * send it messages, and use glimslog.log(char * msg)
 * or glimslog.log(string msg)
 * to add a new message to the queue.
 *
 * example of usage to print some information plus the
 * of x to the log:
 *
 *  extern TraceLog glimslog ;
 *  std::stringstream logstream ;
 *  logstream << "Message text " << x << endl ;
 *  glimslog.log( logstream.str() ) ;
 *
 *
 * @author Deborah Lee Soltesz
 *
 * @history 2005-05-13 Created new TraceLog class
 * @history 2005-07-14 New function for storing strings
 * @history 2005-07-14 Added new functions for printing
 *                     log as messages come in, with or
 *                     without storing messages in memory
 */

//static
class TraceLog {
protected:
  fstream logFile ;
  std::vector<string> messages ;
  int lastPrint ;
  time_t lasttime ;
  bool timing ;

public:
  TraceLog() {
    lastPrint = 0 ;
    time (&lasttime) ;

    // *** EDIT HERE
    // set to true to enable output of timing
    timing = false ;

    // *** EDIT HERE
    // uncomment the following line to enable this functionality
//    logFile.open ("glimsview_log.txt", fstream::out) ;
  }

  virtual ~TraceLog() {
  }

  /**
   * Add a message to the list of messages to be written to the
   * log file when the application exits.
   */
  void log (char * msg) {
    if (logFile.is_open()) {
      time_t curtime ;
      time (&curtime) ;
      double dtime = difftime (curtime, lasttime) ;

      std::stringstream msgstream ;
      if (timing) {
        msgstream << ctime(&curtime) << " ("
                  << dtime << ")    "
                  << msg << endl ;
      }
      else {
        msgstream << msg ;
      }
//      messages.push_back (msgstream.str()) ;
//      writeLogNow () ; // enable for getting trace on crashing bugs
      writeLogRightNow (msgstream.str()) ; // enable for getting trace on crashing bugs w/o saving history in memory
      lasttime = curtime ;
    }
  }

  /**
   * Add a message to the list of messages to be written to the
   * log file when the application exits.
   */
  void log (string msg) {
    if (logFile.is_open()) {
      time_t curtime ;
      time (&curtime) ;
      double dtime = difftime (curtime, lasttime) ;

      std::stringstream msgstream ;
      if (timing) {
        msgstream << ctime(&curtime) << " ("
                  << dtime << ")\n  "
                  << msg << endl ;
      }
      else {
        msgstream << msg ;
      }
//      messages.push_back (msgstream.str()) ;
//      writeLogNow () ; // enable for getting trace on crashing bugs
      writeLogRightNow (msgstream.str()) ; // enable for getting trace on crashing bugs w/o saving history in memory
      lasttime = curtime ;
    }
  }

  /**
   * Write the messages to the file and close the file.
   */
  void writeLog () {
    if (logFile.is_open()) {
      logFile << endl << "GLIMSView Log File" << endl ;
      logFile << "Num Msg: " << (int)messages.size() << endl ;
      for (int i = 0 ; i < (int)messages.size() ; i++ ) {
        logFile << i << "  " << messages[i] << endl ;
      }
      logFile.close() ;
    }
  }

  /**
   * Write the latest messages to the file.
   */
  void writeLogNow () {
    if (logFile.is_open()) {
      int i = 0 ;
      for ( i = lastPrint ; i < (int)messages.size() ; i++ ) {
        logFile << i << "  " << messages[i] << endl ;
      }
      logFile << flush ;
      lastPrint = i ;
    }
  }

  /**
   * Write the messages to the file without saving history in memory.
   */
  void writeLogRightNow (string msg) {
    if (logFile.is_open()) {
      logFile << msg << endl ;
      logFile << flush ;
    }
  }

};


#endif // !defined(_TRACELOG_H_)



