#ifndef __IMAGECUBE_H
#define __IMAGECUBE_H

#include <vector>
#include "ImageInterface.h"

/**
 *  ImageCube is a representation of an image in an easy-to-manage form.
 *  The general purpose of ImageCube is to hold a part of an image, but
 *  it could hold the whole image.  Pixels can be refereced by xyz
 *  coordinates relatice to both image and cube.
 *
 *  @author Erica Liszewski 2006-04-10
 */
class ImageCube {

public:

  //! ThreeDPoint - a point in 3d space
  typedef struct{
    double x;
    double y;
    int z;
  } ThreeDPoint;

  //! Pixel - a pixel
  typedef struct{
    int x;
    int y;
  } Pixel;

protected:

/**
 *   Layer - represents an image band
 */
  typedef struct{
    int layernum;
    std::vector<double> * values;
  } Layer ;

  ImageInterface* image;
  int width;                //!< width of the cube
  int height;               //!< height of the cube
  int layers;               //!< number of layers in the cube
  ThreeDPoint lowestPoint;  //!< where (0,0,0) falls in the original image
  ThreeDPoint highestPoint; //!< where the opposite corner falls in original image

  std::vector<Layer> theCube;//!< holds all the layers, effectivly the cube

public:

  ImageCube(int x1, int y1, int w1, int h1, ImageInterface* theimage);

/**
 *    constructor, creates a cube and sets the size
 *    @param [in] x1 int where the lowest corner is in the original image
 *    @param [in] y1 int where the lowest corner is in the original image
 *    @param [in] w1 int the width of the cube
 *    @param [in] h1 int the height of the cube
 */
  ImageCube(int x1, int y1, int w1, int h1);

/**
 *   destructor
 */
  ~ImageCube();

  /**
   *  int z - z index (band or layer)
   *  returns the value for layer (band) z
   */
  std::vector<double> * getValuesOf(int z) ;

  /**
   *  Uses a local reference (i.e. starts at 0,0).
   *  @param [in] x int x index
   *  @param [in] y int y index
   *  @param [in] z int z index
   *  @return double the value at the location specified.
   */
  double getValueAtLocal(int x, int y, int z);


/**
 *   Location is specified in terms of the original
 *   image.
 *   @param [in] x int x index
 *   @param [in] y int y index
 *   @param [in] z int z index
 *   @return double the value at the location specified,
 */
  double getValueAtOverall(int x, int y, int z);

/**
 *   @return ThreeDPoint corresponds to where
 *   the lowest point in this cube is in the original
 *   image.
 */
  ThreeDPoint getLowestPoint();

/**
 *   @return ThreeDPoint corresponds to where
 *   the highest point in this cube is in the original
 *   image.
 */
  ThreeDPoint getHighestPoint();

/**
 *   adds a layer to the end of the current "stack" of layers
 *   @param [in] vals std::vector<double>  the values to form the layer
 *   @returns boolean true if successful, and false if not
 */
  bool addBand(std::vector<double>* vals);

/**
 *   add a layer in to a specific location
 *   @param [in] vals std::vector<double>  the values to form the layer
 *   @param [in] layernum int the location of the layer
 *   @returns boolean true if successful, and false if not
 */
  bool addBand(std::vector<double>* vals, int layernum);

/**
 *   adds several layers in consecutive order.  The size of
 *   the cube is determined when initialized, so we can
 *   determine each layer from the next based on size
 *   @param [in] vals std::vector<double>  the values to form the layer
 *   @returns boolean true if successful, and false if not
 */
  bool addBands(std::vector<double>* vals);

};

#endif


