/**
 * ImageInterface provides a combined and more useful/intuitive (at
 * least for me) way of accessing GLIMSview images.  The various image
 * classes seemed too fragmented and individually speciallized for
 * easy use, so this class attempts to unify and abstract some of
 * the common functions into one class.  This class uses the Singleton
 * pattern, so there's only ever one of them.  You must use getInterface()
 * to get a pointer the the interface.

 * @author Erica Liszewski 2006-03-10
 */

#ifndef __IMAGEINTERFACE_H
#define __IMAGEINTERFACE_H

#include "glimsproject.h"

#include <qimage.h>


class ImageInterface {

protected:

  static bool instanceFlag;//true if an instance has been made
  static ImageInterface* instance;//pointer to the instance, if there is one

  /**
   *  ImageInterface( GLIMSProject * proj)
   *  GLIMSProject* proj - the GLIMSProject this class is attached too
   * creates an ImageInterface
   */
  ImageInterface( GLIMSProject *proj );

  GLIMSProject *mProj;  //the project

  /**
   * Image* getImage()
   * get the Image associated with this project, this is needed
   * to to the translation of functions.
   */
  Image* getImage();

  /**
   * ImageFormat* getImageFormat()
   * the ImageFormat is needed for info about the image
   */
  ImageFormat* getImageFormat();

  /**
   * std::vector<double> getValueAsDouble(void* vals, int bandnum, int width);
   *  void* vals - a pointer to an array of values in any format
   *  int bandnum - the band (layer) that these values are taken from
   *  int width - the length of the array vals
   * returns a vector of double values representing an area in the image.
   * all data types are translated to doubles to provide a unified face
   * to the data.  This pointer is created with new, so you must
   * ensure that it is properly destroyed.
   */
  std::vector<double>* getValueAsDouble(void* vals, int bandnum, int width);

public:



  /**
   * static ImageInterface* getInterface(GLIMSProject* proj)
   *  GLIMSProject* proj - the project to be associated with
   * this class will return a pointer to the instance of the
   * ImageInterface.  If no ImageInterface has been created, it
   * creates one and returns a pointer.
   */
  static ImageInterface* getInterface(GLIMSProject* proj);

  static void destroyInterface();

  ~ImageInterface();

  /**
   * std::vector<double> getCubeLatLong(double lat1, double lon1, int z1, double lat2, double lon2, int z2);
   *  double lat1 - the lower lattitude
   *  double lon1 - the lower longitude
   *  int z1 - the lowest band
   *  double lat2 - the highest lattitude
   *  double lon2 - the highest longitude
   *  int z2 - the higest band
   * return a vector of doubles of the values from the image.  Exactly like
   * getCube, except uses lat/lon instead of x/y.  This pointer is
   * created with new, so you must ensure that it is destroyed properly.
   */
  std::vector<double>* getCubeLatLong(double lat1, double lon1, int z1, double lat2, double lon2, int z2);

  /**
   * std::vector<double> getCube(int x1, int y1, int z1, int x2, int y2, int z2)
   *  int x1 - the beginning x value of the footprint of the cube
   *  int y1 - the beginning y value of the footprint of the cube
   *  int z1 - the upper or lowest layer to include
   *  int x2 - the ending x value of the footprint of the cube
   *  int y2 - the ending y value of the footprint of the cube
   *  int z2 - the upper or lower (opposite of z1) layer to include
   * returns a vector of doubles representing a cube from the image.
   * the x and y values are used to determine a retangular footprint,
   * while the z values determine the layer to use.  This pointer is
   * created with new, so you must ensure that it is destroyed properly.
   */
  std::vector<double>* getCube(int x1, int y1, int z1, int x2, int y2, int z2);

  /**
   * std::vector<double> getCubeAllBands(int x1, int y1, int x2, int y2)
   *  int x1 - an x value for the footprint
   *  int y1 - a y value for the footprint
   *  int x2 - the other x value for the footprint
   *  int y2 - the other y value for the footprint
   * returns a vector of doubles from the image.  the x and y values
   * determine an rectangular footprint, and all layers are returned.
   * This pointer is  created with new, so you must ensure that it
   * is destroyed properly.
   */
  std::vector<double>* getCubeAllBands(int x1, int y1, int x2, int y2);

  /**
   * std::vector<double> getSlice(int x1, int y1, int z1, int x2, int y2, int z2);
   *  int x1 - x value of fisrt corner
   *  int y1 - y value of fisrt corner
   *  int z1 - laver of first corner
   *  int x2 - x value of corner diagnal accross plane
   *  int y2 - y value of corner diagnal accross plane
   *  int z2 - z value of corner diagnal accross plane
   * returns a vector of double  from the image.  The values are from a
   * slice (or plane) of any size in any major direction (i.e. you can do
   * slice along any of the x, y, or z axis), but not diagonal.  This
   * pointer is created with new, so you must ensure that it is
   * destroyed properly.
   */
  std::vector<double>* getSlice(int x1, int y1, int z1, int x2, int y2, int z2);


  /**
   * std::vector<double> getSliceScaled(int x1, int y1, int z1, int x2, int y2, int z2);
   *  int x1 - x value of fisrt corner
   *  int y1 - y value of fisrt corner
   *  int z1 - laver of first corner
   *  int x2 - x value of corner diagnal accross plane
   *  int y2 - y value of corner diagnal accross plane
   *  int z2 - z value of corner diagnal accross plane
   * returns a vector of double  from the image.  The values are from a
   * slice (or plane) of any size in any major direction (i.e. you can do
   * slice along any of the x, y, or z axis), but not diagonal.  If this
   * slice lies on a layer that is not the same size as the first layer
   * the slice is scaled to the size of the first layer.  This
   * pointer is created with new, so you must ensure that it is
   * destroyed properly.
   */
  std::vector<double>* getSliceScaled(int x1, int y1, int z1, int x2, int y2, int z2);


  /**
   * std::vector<double> getPoint(int x1, int y1, int z1)
   *  int x1 - x value of the point
   *  int y1 - y value of the point
   *  int z1 - z value of the point
   * returns a double of the value at a single point in the image.
   */
  std::vector<double>* getPoint(int x1, int y1, int z1);

  /**
   * int getBands()
   * returns the number of layers (bands) in the image
   */
  int getBands();

  /**
   * int getHeight()
   * returns the number of pixels in the y direction
   */
  int getHeight();

  /**
   * int getWidth()
   * return the number of pixels in the x direction
   */
  int getWidth();

  /**
   * char* getPath()
   * returns the file path to the image (I hope)
   */
  char* getPath();

  /**
   * QImage getQImage(int neww, int newh)
   *  int neww - the width of the returned image
   *  int newy - the height of the returned image
   * returns a QImage of the specified height.  If the specified
   * height is different from the original height, the image
   * is scaled accordingly.
   */
  QImage* getQImage(int neww, int newh);

  /**
   * QImage* getQImageArea(int x, int y, int w, int h, int neww, int newh)
   *  int x - lowest x value
   *  int y - lowest y value
   *  int w - width in original image
   *  int h - height in original image
   *  int neww - width of returned image
   *  int newh - height of returned image
   * returns a QImage that is a subsection of the original image.
   * the subsection starts at x, y, and is (w x h) pixels.  If
   * neww and newh are different from w and h, then the image
   * is scaled accordingly.
   */
  QImage* getQImageArea(int x, int y, int w, int h, int neww, int newh);

  /**
   * std::vector<std::string> getBandNames()
   * returns a vector of the band names in order
   */
  std::vector<std::string> getBandNames();

  /**
   * void getLL( int band, &double x, &double y)
   *  int band - the band to use for lat/lon referance
   *  &double x - the input x, will be output lat
   *  &double y - the input y, will be output lon
   * Accesor for ImageFormat::getLL.
   * Translates the input XY values into the corresponding Lat/Lon
   */
  void getLL( int band, double &x, double &y);

  /**
   * void getXY( int band, &double lat, &double lon)
   *  int band - the band to use for lat/lon referance
   *  &double lat - the input lat, will be output x
   *  &double lon - the input lon, will be output y
   * Accesor for ImageFormat::getXY.
   * Translates the input Lat/Lon values into the corresponding XY
   */
  void getXY( int band, double &lat, double &lon);

  /**
   * std::string runDriverTest()
   * runs a driver that creates test ImageInterface and adds
   * values to them.  The returned string are results used
   * to deterine proper function.
   */
  std::string runDriverTest();
};

#endif

