#ifndef __GLACIERIDDEF_H
#define __GLACIERIDDEF_H

#include "shape.h"
#include "glimsglobals.h"
#include <string>


/**
 * This holds data that represents a glacier id in accordance
 * with the GLIMS specifications.  XML io routines are provided
 * for easy serialization.  A routine for producing a string
 * representation in accordance with the GLIMS specification is
 * provided.
 */

class GlacierIDDef : public XMLSerializeable {
public:
  double      mLat;          //!<  LATITUDE OF GLACIER ID
  double      mLon;          //!<  LONGITUDE OF GLACIER ID
  Node        mImgCoord;     //!<  COORDINATE WITHIN IMAGE
  std::string mName;         //!<  NAME OF GLACIER
  std::string mWGMSID;       //!<  WGMS ID
  std::string mLocalID;      //!<  LOCAL(TO RC) ID
  std::string mParentID;     //!<  PARENT GID
  int         mPrimCsfn;     //!<  PRIMARY CLASSIFICATION INDEX
  int         mForm;         //!<  FORM INDEX
  int         mFChar;        //!<  FRONTAL CHARACTERISTICS INDEX
  int         mLChar;        //!<  LONGITUDINAL CHARACTERISTICS INDEX
  int         mDomMassSrc;   //!<  DOMINANT MASS SOURCE INDEX
  int         mTongueAct;    //!<  TOUNGE ACTIVITY INDEX
  float       mWidth;        //!<  WIDTH
  float       mLength;       //!<  LENGTH
  float       mArea;         //!<  AREA
  float       mAbzoneArea;   //!<  ABZONE AREA
  float       mSpeed;        //!<  SPEED
  float       mELA;          //!<  ELA
  std::string mELADesc;      //!<  ELA DESCRIPTION
  float       mSnowLineElev; //!<  SNOW LINE ELEVATION
  std::string mRecordStatus; //!<  RECORD STATUS

  GlacierIDDef( ) {
    mLat          = 0.0;   // LATITUDE OF GLACIER ID
    mLon          = 0.0;   // LONGITUDE OF GLACIER ID
    mName         = "";    // NAME OF GLACIER
    mWGMSID       = "";    // WGMS ID
    mLocalID      = "";    // LOCAL(TO RC) ID
    mParentID     = "";    // PARENT GID
    mPrimCsfn     = -1;    // PRIMARY CLASSIFICATION INDEX
    mForm         = -1;    // FORM INDEX
    mFChar        = -1;    // FRONTAL CHARACTERISTICS INDEX
    mLChar        = -1;    // LONGITUDINAL CHARACTERISTICS INDEX
    mDomMassSrc   = -1;    // DOMINANT MASS SOURCE INDEX
    mTongueAct    = -1;    // TOUNGE ACTIVITY INDEX
    mWidth        = 0.0f;  // WIDTH
    mLength       = 0.0f;  // LENGTH
    mArea         = 0.0f;  // AREA
    mAbzoneArea   = 0.0f;  // ABZONE AREA
    mSpeed        = 0.0f;  // SPEED
    mELA          = 0.0f;  // ELA
    mELADesc      = "";    // ELA DESCRIPTION
    mSnowLineElev = 0.0f;  // SNOW LINE ELEVATION
    mRecordStatus = "";    // RECORD STATUS
  }

  ~GlacierIDDef( ) {;}

  virtual bool fromXML( QDomElement &elem ) {
    if ( std::string( "GlacierIDDef" ).compare( (const char*)elem.tagName() ) )
      return false;

    QDomElement imgcoordelem = getElemByTagName( elem, "ImgCoord" );
    mLat =          readDoubleTag( elem, "Lat" );
    mLon =          readDoubleTag( elem, "Lon" );

    mName =         readTextTag( elem, "Name" );
    mWGMSID =       readTextTag( elem, "WGMSID" );
    mLocalID =      readTextTag( elem, "LocalID" );
    mParentID =     readTextTag( elem, "ParentID" );

    mPrimCsfn =     readIntTag( elem, "PrimCsfn" );
    mForm =         readIntTag( elem, "Form" );
    mFChar =        readIntTag( elem, "FChar" );
    mLChar =        readIntTag( elem, "LChar" );
    mDomMassSrc =   readIntTag( elem, "DomMassSrc" );
    mTongueAct =    readIntTag( elem, "TongueAct" );

    mWidth =        readDoubleTag( elem, "Width" );
    mLength =       readDoubleTag( elem, "Length" );
    mArea =         readDoubleTag( elem, "Area" );
    mAbzoneArea =   readDoubleTag( elem, "AbzoneArea" );
    mSpeed =        readDoubleTag( elem, "Speed" );
    mELA =          readDoubleTag( elem, "ELA" );
    mELADesc =      readTextTag( elem, "ELADesc" );
    mSnowLineElev = readDoubleTag( elem, "SnowLineElev" );
    mRecordStatus = readTextTag( elem, "RecordStatus" );

    return true;
  }

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) {
    QDomElement base = doc.createElement( "GlacierIDDef" );
    if ( id != "" )
      base.setAttribute( "id", (char*)id.c_str() );
    elem.appendChild( base );

    writeDoubleTag( doc, base, "Lat", mLat );
    writeDoubleTag( doc, base, "Lon", mLon );
    writeTextTag( doc, base, "Name", mName );
    writeTextTag( doc, base, "WGMSID", mWGMSID );
    writeTextTag( doc, base, "LocalID", mLocalID );
    writeTextTag( doc, base, "ParentID", mParentID );
    writeIntTag( doc, base, "PrimCsfn", mPrimCsfn );
    writeIntTag( doc, base, "Form", mForm );
    writeIntTag( doc, base, "FChar", mFChar );
    writeIntTag( doc, base, "LChar", mLChar );
    writeIntTag( doc, base, "DomMassSrc", mDomMassSrc );
    writeIntTag( doc, base, "TongueAct", mTongueAct );
    writeDoubleTag( doc, base, "Width", mWidth );
    writeDoubleTag( doc, base, "Length", mLength );
    writeDoubleTag( doc, base, "Area", mArea );
    writeDoubleTag( doc, base, "AbzoneArea", mAbzoneArea );
    writeDoubleTag( doc, base, "Speed", mSpeed );
    writeDoubleTag( doc, base, "ELA", mELA );
    writeTextTag( doc, base, "ELADesc", mELADesc );
    writeDoubleTag( doc, base, "SnowLineElev", mSnowLineElev );
    writeTextTag( doc, base, "RecordStatus", mRecordStatus );
    return true;
  }

  // updateVersion: make applicable fixes to the glacier info in the
  //                project based on the version date of the project file
  virtual void updateVersion (QDate * projectVersion) {

    // 5/20/20034: classifications off by 1 - stored as reference to index of dropdown box
    //             instead of the WGMS specification number
    QDate checkDate (2004, 5, 20) ;
    if ( *projectVersion < checkDate ) {
      mPrimCsfn   -= 1 ;
      mForm       -= 1 ;
      mFChar      -= 1 ;
      mLChar      -= 1 ;
      mDomMassSrc -= 1 ;
      mTongueAct  -= 1 ;
    }
  }


  // create a glacier ID string from the lat-lon value
  std::string toString( ) {
    std::string     gid;        //!<  THE GID TO BE RETURNED
    char            buf[50];    //!<  STRING BUFFER
    std::string     northing;   //!<  'N' OR 'S'
    std::string     flonpad;    //!<  HOLDS PRECEDING ZEROS FOR LON
    std::string     flatpad;    //!<  HOLDS PRECEDING ZEROS FOR LAT
    std::string     blonpad;    //!<  HOLDS FOLLOWING ZEROS FOR LON
    std::string     blatpad;    //!<  HOLDS FOLLOWING ZEROS FOR LAT
    int             latprec;    //!<  LENGTH OF LAT PRECISION
    int             lonprec;    //!<  LENGTH OF LON PRECISION

    double          tLat = mLat ;      // temp container for latitude
    double          tLon = mLon ;      // temp container for latitude

    // ---------------------------------------------------------
    // DETERMINE NORTHING STRING
    if ( tLat < 0 ) {
      northing = "S";
      tLat *= -1;
    } else {
      northing = "N";
    }
    // ---------------------------------------------------------

    // OFFSET LON
    if ( tLon < 0 )
      tLon += 360;

    // DETERMINE LAT/LON PRECISION
    latprec = (int)((tLat - (int)tLat) * 1000.0 + 0.5);
    lonprec = (int)((tLon - (int)tLon) * 1000.0 + 0.5);

    // ---------------------------------------------------------
    // FILL PADDINGS
    if ( tLon < 10 )
      flonpad = "00";
    else if ( tLon < 100 )
      flonpad = "0";

    if ( tLat < 10 )
      flatpad = "0";

    if ( latprec < 10 )
      blatpad = "00";
    else if ( latprec < 100 )
      blatpad = "0";

    if ( lonprec < 10 )
      blonpad = "00";
    else if ( lonprec < 100 )
      blonpad = "0";
    // ---------------------------------------------------------

    // LOAD BUFFER
    sprintf( buf,
             "G%s%d%s%dE%s%d%s%d%s",
             flonpad.c_str(),
             (int)tLon,
             blonpad.c_str(),
             lonprec,
             flatpad.c_str(),
             (int)tLat,
             blatpad.c_str(),
             latprec,
             northing.c_str() );

    gid = buf;
    return gid;
  }
};

#endif


