#ifndef GLACIERIDDLG_H
#define GLACIERIDDLG_H

// QT
#include <qdialog.h>
#include <qevent.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qlabel.h>

// LOCAL
#include "glimsglobals.h"
#include "glimsgiddata.h"

#include <cstdlib>

/**
 * This is a form allowing the user to enter data
 * associated with glacier IDs.  This is not a modal
 * dialog, but instead lets the user track lat/lon
 * on the image while this is open.  F1 can be pressed
 * while this is focused to capture lat/lon and they will be
 * placed into the lat/lon field in this form.
 *
 * The information specified in this dialog by the user
 * corresponds to the "glaciers" Shapefile Attributes
 * specified in the GLIMS Data Transfer Specification
 * (http://www.glims.org/MapsAndDocs/datatransfer/data_transfer_specification.html).
 * Valid values for these attributes can be found in
 * the "valid values" document (http://www.glims.org/MapsAndDocs/datatransfer/valids.txt)
 *
 */

class GlacierIDDlg : public QDialog {
  Q_OBJECT
  protected:
  int             miGID;               //!< Unique GID ID object that stores this glacier's information.
  GLIMSGIDData    &mGIDData;           //!< Ptr to the linetool.
  QLineEdit       *mtxtLat;            //!< Latitude.
                                       /*!< Widget to enter/display the latitude in decimal degrees; positive=north, negative=south. */
  QLineEdit       *mtxtLon;            //!< Longitude.
                                       /*!< Widget to enter/display the longitude in decimal degrees; positive=east, negative=west. */
  QLineEdit       *mtxtName;           //!< Name.
                                       /*!< Widget to enter/display the name of the glacier, if one exists. */
  QLineEdit       *mtxtWGMSID;         //!< WGMS ID.
                                       /*!< Widget to enter/display the World Glacier Monitoring Service (WGMS) ID. The WGMS ID is a unique ID assigned to the glacier based on a variety of information, specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QLineEdit       *mtxtLocalID;        //!< Local (RC) ID.
                                       /*!< Widget to enter/display the GLIMS Regional Center ID. */
  QLineEdit       *mtxtParentIMID;     //!< Parent GID.
                                       /*!< Widget to enter/display the local ID. */
  /*
  QLineEdit       *mtxtDataSource;
  QLineEdit       *mtxtFName;
  QLineEdit       *mtxtLName;
  QLineEdit       *mtxtProcDesc;
  QLineEdit       *mtxt3dDesc;
*/

  QComboBox       *mcmbPrimCsfn;     //!< Primary classification.
                                     /*!< Widget to enter/display the primary classification (type of glacier) specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QComboBox       *mcmbForm;         //!< Form.
                                     /*!< Widget to enter/display the Widget to enter/display the form (shape of the glacier) specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QComboBox       *mcmbFrntChar;     //!< Frontal characteristics.
                                     /*!< Widget to enter/display the frontal characteristics  of the glacier specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QComboBox       *mcmbLonChar;      //!< Longitudinal characteristics.
                                     /*!< Widget to enter/display the longitudinal profile of the glacier specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QComboBox       *mcmbDomMassSrc;   //!< Dominant mass source.
                                     /*!< Widget to enter/display the major source of nourishment of the glacier specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QComboBox       *mcmbTongueAct;    //!< Tounge activity.
                                     /*!< Widget to enter/display the activity of the tongue of the glacier specified at http://nsidc.org/data/docs/noaa/g01130_glacier_inventory/. */
  QLineEdit       *mtxtWidth;        //!< Width.
                                     /*!< Widget to enter/display the width of the glacier, in kilometers. */
  QLineEdit       *mtxtLength;       //!< Length.
                                     /*!< Widget to enter/display the length of the glacier, in kilometers. */
  QLineEdit       *mtxtArea;         //!< Area.
                                     /*!< Widget to enter/display the area of the glacier, in square kilometers. */
  QLineEdit       *mtxtAbzoneArea;   //!< Abzone area.
                                     /*!< Widget to enter/display the area of the ablation zone, in square kilometers. */
  QLineEdit       *mtxtSpeed;        //!< Speed.
                                     /*!< Widget to enter/display the speed (?). */
  QLineEdit       *mtxtELA;          //!< ELA.
                                     /*!< Widget to enter/display the ELA (?). */
  QLineEdit       *mtxtELADesc;      //!< ELA description.
                                     /*!< Widget to enter/display the the ELA (?) description. */
  QLineEdit       *mtxtSnowLineElev; //!< Snow line elevation.
                                     /*!< Widget to enter/display the snow line elevation, in meters above sea level. */
  QLineEdit       *mtxtRecordStatus; //!< Record status.
                                     /*!< Widget to enter/display the status of this record. */

/**
 * This uses the valids available from LineTool for the
 * combo boxes.
 *
 */
  void loadValids( );

/**
 * Fill the form with glacier information from the selected GID .
 *
 */
  void loadGID( );

/**
 * Initializes the form.
 *
 */
  void initialize( );

public:

/**
 * Initializes the form.
 *
 */
  GlacierIDDlg( GLIMSGIDData & );

/**
 * Initializes the form.  This constructor takes an index
 * of a GID and fills the form with this GID for editing.
 *
 */
  GlacierIDDlg( GLIMSGIDData &, int );
  ~GlacierIDDlg( );

/**
 * Event handler for the key press event.  This routine
 * allows the user to capture lat lon from mouse coords
 * of the views while this window is focused.
 *
 */
  void keyPressEvent( QKeyEvent * );

public slots:
/**
 * Event handler for the ADD button.  This will capture
 * the data in the form and fill a new GID with it.  Then
 * either add or overwrite to the GID set in LineTool.
 *
 */
  void addClicked( );

/**
 * Event handler for the CANCEL button.
 *
 */
  void cancelClicked( );

  signals:
  void gidSetChanged( );

};

#endif



