#ifndef __GLIMSGIDDATA_H
#define __GLIMSGIDDATA_H

#include "simplepointdata.h"
#include "glacieriddef.h"
#include "image.h"
#include <vector>
#include <stack>

/**
 * Point vector data specifically representing glacier ids
 * described in the GLIMS specifications.  A glacier id is a
 * single point within a glacier's boundaries that is universally
 * unique to this glacier within the GLIMS database.  Extending
 * SimplePointData, this is also an XMLSerializable object.
 *
 * @author Dan Mingus
 * @history 2005-07-23 Deborah Lee Soltesz
 *          Added ability to redo undone actions (i.e. popped data).
 *          New functions for managing data during undo/redo.
 * @history 2005-08-04 Deborah Lee Soltesz
 *          Added undo/redo for mDefSet (set of glacier definitions
 *          which hold the glacier attribute metadata)
 *
 * @whatsnew 2005-07-23 Deborah Lee Soltesz
 *                      Redo!!!
 *
 */

class GLIMSGIDData : public SimplePointData {
protected:
  std::vector<GlacierIDDef>   mDefSet;

  std::stack < std::vector <GlacierIDDef> > mUndoGIDDef ;
  std::stack < std::vector <GlacierIDDef> > mRedoGIDDef ;

  /**
   * Put current dataset on redo stack
   *
   * @author 2005-07-23 Deborah Lee Soltesz
   */
  virtual void pushRedo ( );

  /**
   * Clear out current data set
   *
   * @author 2005-07-23 Deborah Lee Soltesz
   */
  virtual void clear( );

  /**
   * Clear out redo set
   *
   * @author 2005-07-23 Deborah Lee Soltesz
   */
  virtual void clearRedo( );

public:
  GLIMSGIDData( ) {
  }
  virtual ~GLIMSGIDData( ) {
  }

  virtual bool fromXML( QDomElement &elem );
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id="" );

  virtual void addNode( Node &node, int inode );
  virtual void delNode( int inode );
  virtual void moveNode( Node &node, int inode = -1 );
  virtual void insertNode( Node &node, int inode );

  /**
   * Put history information on undo stacks
   */
  virtual void push( );

  /**
   * Copies on-screen selection to undo stack
   *
   * @author 2005-07-06 Deborah Lee Soltesz
   */
  virtual void pushSelection( ) {;}

  /**
   * Undo: Sets current state to most recent
   *       state stored on undo stack, pushes
   *       old state onto redo stack.
   */
  virtual void pop( );

  /**
   * Red: Sets current state to most recent
   *      state stored on redo stack, pushes
   *      old state onto undo stack
   *
   * @author 2005-07-23 Deborah Lee Soltesz
   */
  virtual void redo( );

  void addGID( const GlacierIDDef &gid );

  /**
   * fixes errors and makes changes to project file
   * based on the project's version date
   */
  void updateVersion (QDate * projectVersion) ;

  std::vector<GlacierIDDef>& getDefSet( ) {
    return mDefSet ;
  }

};

#endif



