#ifndef __GLIMSLINEDATA_H
#define __GLIMSLINEDATA_H

#include <vector>
#include <algorithm>
#include "image.h"
#include "xmlserializeable.h"
#include "simplelinedata.h"
#include "linedefset.h"
#include "glacierline.h"
#include "GlacierLineHistory.h"

/**
 * This holds all glacier outlines and maps to them a
 * GID and a LDF.  These mappings are integer indices
 * into LineDefSet which is local and the set of GIDs
 * that is stored in GLIMSGIDData.  This provides
 * routines for modifying the lines it contains as
 * well as routines for bulk import.
 *
 * @history 2005-07-06 Deborah Lee Soltesz
 *                     Refactored undo functionality
 * @history 2005-07-23 Deborah Lee Soltesz
 *                     Added redo, made a generic pop function
 *                     for new redo functionality
 *
 * @whatsnew 2005-07-23 Deborah Lee Soltesz
 *                      Redo!!!
 *
 * @todo     2005-07-12 Deborah Lee Soltesz
 *                      Eliminate redundant "with and
 *                      without history" functions as much
 *                      as possible. See also GlacierLineHistory.
 * @todo     2005-07-23 Deborah Lee Soltesz
 *                      Is memory being properly cleaned up during
 *                      undo and redo? Are GID/Line associations
 *                      properly managed?
 *
 */

class GLIMSLineData
: public SimpleLineData {

protected:
  int         mDefGID;  //!< Default Glacier ID
  int         mDefLDF;  //!< Default Line Definition
  LineDefSet  mLDFSet;  //!< Set of Line Definitions

  GlacierLineHistory mHistory ;     //!< History of actions taken on GlacierLines
  GlacierLineHistory mRedoHistory ; //!< History of actions popped off the history stack

  void setAttr( int iline );

 /**
  * Generic Un/Redo for GLIMSLineData implemented by making the top of the "undo" stack
  * the current state, popping the stacks so the top of the stacks are the previous state
  * and pushing that to the "redo" stack. Whether this is an "undo" or a "redo" depends
  * on the order the mHistory and mRedoHistory stacks (and associated selection set stacks)
  * are put into the function parameters.
  *
  * Usage
  *   - undo: pop (&mHistory, &mRedoHistory, &mUndoSel, &mRedoSel) ;
  *   - redo: pop (&mRedoHistory, &mHistory, &mRedoSel, &mUndoSel) ;
  *
  * Actions stored in doubles (e.g. join line, break line) will have their two
  * elements reversed when they're pushed to "redo" stack so that the order of
  * the elements is maintained no matter what.
  *
  * @param historyUndo The history stack to be popped
  * @param historyRedo The history stack to have the popped history pushed to
  * @param selSetUndo  The selection stack to popped
  * @param selSetRedo  The selection stack to have the popped selection pushed to
  */
  void pop( GlacierLineHistory       * historyUndo, GlacierLineHistory       * historyRedo,
            std::stack<SelectionSet> * selSetUndo,  std::stack<SelectionSet> * selSetRedo) ;


public:
  GLIMSLineData( );

  /**
   * Write Glacier Line information to the XML project file
   */
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" );

  /**
   * Read Glacier Line information from XML project file
   */
  virtual bool fromXML( QDomElement &elem );

  /**
   * Set default Glacier ID
   */
  void setDefGID( int gid ) {
    mDefGID = gid;
  }

  /**
   * Set default Line Definition
   */
  void setDefLDF( int ldf ) {
    mDefLDF = ldf;
  }

  /**
   * Set Line Definition at index ildf
   * in Line Definition Set to be associated
   * to the line at index iline in the
   * mXYSet and mLLSet
   *
   */
  void setLDF( int iline, int ildf );

  /**
   * Set the Glacier ID at index igid
   * to be associated to the line at
   * index iline in the mXYSet and mLLSet
   */
  void setGID( int iline, int igid );

  /**
   * Create a new GlacierLine using
   * the Node as the starting point
   *
   *@param xynode The starting point of the line,
   *              containing the XY coordinates of
   *              the image where the user clicked
   *
   */
  virtual void addLine( Node &xynode );


  /**
   * Restore a deleted GlacierLine
   *
   * @param line  A line containing nodes with XY
   *              coordinates. Lat/Lon coordinates
   *              will be calculated as the line
   *              is added to the line set. No
   *              history information will be
   *              recorded by this function.
   *
   * @param iline Position in the line set to
   *              insert the line. If the default
   *              of -1 is taken, the line will be
   *              added to the end of the line set.
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   *
   */
  virtual void restoreLine( GlacierLine &line, int iline = -1 ) ;

  /**
   * Delete an entire line
   *
   *@param iline The index of the line of the line to
   *             be deleted in the vector "sets"
   *             of line data mXYSet and mLLSet
   */
  virtual void delLine( int iline );

  /**
   * Remove an entire line without history.
   * Intended for use as an undo to addLine
   *
   *@param iline The index of the line of the line to
   *             be deleted in the vector "sets"
   *             of line data mXYSet and mLLSet
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   *
   */
  virtual void removeLine( int iline ) ;

  /**
   * Break the line at index iline at the node
   * at index inode into two lines. The node
   * at inode will be duplicated as the ending
   * point of one line and the starting point
   * of the next.
   *
   *@param iline The index of the line of the line to
   *             be broken in the vector "sets"
   *             of line data mXYSet and mLLSet
   *
   *@param inode The index of the node where the line
   *             is to be broken
   */
  virtual void breakLine( int iline, int inode );

  /**
   * Undo a joinLine, restoring the original lines.
   * Break the line at index iline at the node
   * at index inode in order to restore the two
   * original lines made into a single line by a join.
   * The node at inode will be duplicated as the ending
   * point of one line and the starting point of the
   * next. No history information is stored. This
   * is intended to be used as an undo function for
   * joinLines.
   *
   * @param line1  Copy of the original line 1 before the join
   *
   * @param line2  Copy of the original line 2 before the join
   *
   * @param iline1 The index of line 1 to be restored. This line
   *               became the joined line.
   *
   * @param iline2 The index of line 2 to be restored. This line
   *               was removed from the active dataset after its
   *               vertices/nodes were added to line 1.
   *
   * @param h1     History associated with line 1
   *
   * @param h2     History associated with line 2
   *
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   *
   * @history 2005-07-22 deborah lee soltesz
   *          Added history elements to allow function to change
   *          the stored shapes for later un/redos
   *
   */
  virtual void restoreJoinedLines(GlacierLine &line1, int iline1,
                                  GlacierLine &line2, int iline2,
                                  GlacierLineHistoryElement * h1,
                                  GlacierLineHistoryElement * h2) ;

  /**
   * Join two lines into a single line.
   * Join the two lines at indices il1 and il2 at
   * the nodes at indices in1 and in2 into a single
   * line
   *
   * @param il1   The index of the first line to be joined
   * @param il2   The index of the second line to be joined
   *
   * @param in1   The index of the node in the first line to be joined
   * @param in2   The index of the node in the second line to be joined
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   *
   */
  virtual void joinLine( int il1, int il2, int in1, int in2 );

  /**
   * Rejoin the two lines at indices il1 and il2 at
   * the nodes at indices in1 and in2 into a single
   * line. No history information is saved. Intended
   * to be used as an undo function for breakLine.
   *
   * @param originalLine The original line before modified by the break
   *
   * @param il1   The index of the original line, modified by break
   * @param il2   The index of the new line created from the break
   *
   * @param h1     History associated with line 1
   * @param h2     History associated with line 2
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   *
   * @history 2005-07-22 deborah lee soltesz
   *          Added history elements to allow function to change
   *          the stored shapes for later un/redos
   *
   */
  virtual void rejoinLine( GlacierLine             & originalLine,
                           int                       il1,
                           int                       il2,
                           GlacierLineHistoryElement * h1,
                           GlacierLineHistoryElement * h2 ) ;



  /**
   * Add a Node to the line at index iline.
   * If inode is greater than 0, the Node will
   * be inserted in the line after that index.
   * If inode is -1, the Node will be added to
   * the end of the line.
   *
   * @param node  Node to be added to the line at index iline
   * @param iline Index of the line to be added to
   * @param inode Index of the Node to insert the new Node after
   *
   */
  virtual void addLineNode( Node &node,
                            int iline,
                            int inode = -1 );

  /**
   * Restore a deleted Node to the line at index iline.
   * If inode is greater than 0, the Node will
   * be inserted in the line after that index.
   * If inode is -1, the Node will be added to
   * the end of the line. No history information
   * will be recorded.
   *
   * @param node  Node to be added to the line at index iline
   * @param iline Index of the line to be added to
   * @param inode Index of the Node to insert the new Node after
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   */
  virtual void restoreLineNode( Node &node,
                                int   iline,
                                int   inode ) ;

  /**
   * Delete the Node at index inode in the line at index iline.
   *
   * @param iline The index of the line containing the Node to be removed
   *
   * @param inode The index of the Node to remove
   */
  virtual void delLineNode( int iline, int inode );

  /**
   * Remove the Node at index inode in the line at index iline
   * without recording history information. Intended for use
   * with history to remove an added node.
   *
   * @param iline The index of the line containing the Node to be removed
   *
   * @param inode The index of the Node to remove
   */
  virtual void removeLineNode( int iline, int inode );

  /**
   * Move the Node at index inode in the line at index iline
   * to a new location relative to the base image
   *
   * @param iline The index of the line containing the Node to be moved
   *
   * @param inode The index of the Node to move
   *
   * @param xynode A copy of the node to be moved with modified position
   *               information. The attributes of xynode will be copied
   *               to the node to be moved.
   */
  virtual void moveLineNode( int   iline,
                             int   inode,
                             Node &xynode ) ;

  /**
   * Push history information for moving the Node
   * at index inode in the line at index iline
   * to a new location relative to the base image
   *
   * @param iline The index of the line containing the Node to be moved
   *
   * @param inode The index of the Node to move
   *
   * @param xynode A copy of the original node before it was moved.
   *               The attributes of xynode will be stored before moving.
   */
  virtual void startMoveLineNode( int   iline,
                                  int   inode,
                                  Node &xynode  ) ;

  /**
   * Restores a moved Node at index inode in the line at index iline
   * to its original location, captured in xynode.
   *
   * @param iline The index of the line containing the Node to be moved
   *
   * @param inode The index of the Node to move
   *
   * @param xynode A copy of the original node before it was moved.
   *               The attributes of xynode will be copied to the
   *               node to be restored to its original position.
   */
  virtual void restoreMovedLineNode( int   iline,
                                     int   inode,
                                     Node &xynode ) ;


  /**
   * Insert a Node to the line at index iline.
   * If inode is greater than 0, the Node will
   * be inserted in the line after that index.
   * If inode is -1, the Node will be added to
   * the end of the line.
   *
   * @param node  Node to be inserted to the line at index iline
   * @param iline Index of the line to have a node inserted into
   * @param inode Index of the Node to insert the new Node after
   *
   */
  virtual void insertLineNode( Node &node,
                               int   iline,
                               int   inode=-1 );

  /**
   * Reinsert a deleted Node to the line at index iline.
   * If inode is greater than 0, the Node will
   * be inserted in the line after that index.
   * If inode is -1, the Node will be added to
   * the end of the line. No history information will
   * be recorded.
   *
   * @param node  Node to be inserted to the line at index iline
   * @param iline Index of the line to have a node inserted into
   * @param inode Index of the Node to insert the new Node after
   *
   * @author deborah lee soltesz
   * @date   2005-07-12
   *
   */
  virtual void reinsertLineNode( Node &node,
                               int iline,
                               int inode=-1 );

 /**
  * Undo for GLIMSLineData implemented by making the top of the stack the current state, then
  * popping the stacks so the top of the stacks are the previous state
  */
  virtual void pop( );

 /**
  * Redo for GLIMSLineData implemented by making the moving a history element
  * from the redo stack to the current state and the undo stack
  *
  * @author 2005-07-15 Deborah Lee Soltesz
  */
  virtual void redo( ) ;


 /**
  * Saves the current selection state onto the selection set undo
  * stack as part of history and undo functionality
  */
  virtual void push( ) ;


 /**
  * Causes a history item to be recorded for a user selection/deselection of
  * lines and/or nodes.
  */
  virtual void pushSelection( ) ;


  /**
   * Add a node to line at position inode in line stored at iline,
   * where the node was built using lat-lon instead of XY coordinates.
   * Used by shapefile import funtions.
   */
  virtual void addLineNodeLL( Node &n, int iline, int inode=-1 ) {
    mImg->getXY( n.x, n.y );
    addLineNode( n, iline, inode );
  }

  /**
   * Add a line with a starting node built using lat-lon instead of XY.
   * Used by shapefile import funtions.
   */
  virtual void addLineLL( Node &n ) {
    mImg->getXY( n.x, n.y );
    addLine( n );
  }

  /**
   * Add a line set built using lat-lon instead of XY.
   * Used by shapefile import funtions.
   */
  virtual void addLineSetLL( std::vector<GlacierLine> &glset );

  /**
   * Add a line definition, which contains metadata specific to the feature types
   * it is designed to describe.
   */
  void addLDF( LineDef &ldf ) {
    mLDFSet.add( ldf );
    emit datasetChanged();
  }

  /**
   * Load the default line definition set.
   */
  void loadDefaultSet( ) {
    LineDefSet &ldfset = GLIMSGlobals::mGlbLDFSet;
    for ( unsigned int ildf=0; ildf < ldfset.size(); ildf++ ) {
      addLDF( ldfset[ildf] );
    }
  }

  /**
   * Get the default line definition set.
   */
  LineDefSet& getLDFSet( ) {
    return mLDFSet; emit datasetChanged();
  }

  /**
   * Get the XY coordinate version of the GlacierLine at gl,
   * which is built with lat-lon coordinate information.
   * Used by shapefile import functions.
   */
  GlacierLine* toXY(  GlacierLine *gl );

};


#endif




