#include "glimsproject.h"
#include "dbfheader.h"

GLIMSProject::GLIMSProject( QWidget *parent, ViewSet *vs ) {
  mParent = parent;
  mImg        = NULL;
  mGData      = NULL;
  mLyrSet     = NULL;
  mEditor     = NULL;
  mConfigDlg  = NULL;
  mViewSet    = vs;
  mOpen       = false;
  mNeedSave   = false;
  mFileName   = "";
  GLIMSGlobals::loadGlobals();
  mVersion = new QDate (1900, 1, 1) ;

}

bool GLIMSProject::save( ) {
  if ( mFileName != "" ) {
    mNeedSave = false ;
    return toXMLFile( mFileName, "GLIMS_Project" );
  }
  else {
    return saveAs();
  }
}


bool GLIMSProject::saveAs( ) {
  QString fname = QFileDialog::getSaveFileName();
  if ( fname.isNull() ) {
    return false;
  }

  mFileName = (const char*)fname;
  mNeedSave = false ;
  return toXMLFile( mFileName, "GLIMS_Project" );
}

bool GLIMSProject::create( ) {
  std::string name;
  std::string dir;
  std::string imgname;
  int         type;

  // ProjectDlg *pdlg = new ProjectDlg( name, dir, imgname, type );
  ImageDlg *idlg = new ImageDlg( imgname, type );
  if ( !idlg->exec() )
    return( mOpen = false );

  delete idlg;

  Image *img = Image::openImage( imgname, (Image::MultiFileType)type );
  if ( img == NULL ) {
    QMessageBox::information( NULL, "File Error", "Unable to open selected file" );
    return( mOpen = false );
  }

  // mFileName = name;
  setup( img );
  mGData->getLineData().loadDefaultSet();
  return( mOpen = true );
}

bool GLIMSProject::open( ) {
  QString str = QFileDialog::getOpenFileName();
  if ( str.isNull() ) return( mOpen = false );
  if ( !fromXMLFile( (const char*)str, "GLIMS_Project" ) ) {
    return( mOpen = false );
  }

  mFileName = str.ascii()  ;
  return( mOpen = true );
}

void GLIMSProject::setup( Image *img ) {
  mGData  = new GLIMSDataset( img );
  mLyrSet = new VectorLayerSet;
  mEditor = new VectorEditor( mLyrSet, mViewSet );
  VectorLayer *vl1 = new VectorLayer( &mGData->getLineData() );
  VectorLayer *vl2 = new VectorLayer( &mGData->getGIDData() );
  mLyrSet->addLayer( vl1 );
  mLyrSet->addLayer( vl2 );
  mLyrSet->setActiveLayer( 0 );
  mImg = img;
  mConfigDlg = new ConfigDlg( mParent,
                              *mGData );
  mConfigDlg->hide();
  connect( mConfigDlg,
           SIGNAL( repaintViews() ),
           mViewSet,
           SLOT( repaintViews() ) );

  connect( &mGData->getLineData(),
           SIGNAL( datasetChanged() ),
           this,
           SLOT( setUnsaved() ) );

  connect( &mGData->getGIDData(),
           SIGNAL( datasetChanged() ),
           this,
           SLOT( setUnsaved() ) );

  connect( &(&mGData->getGIDData())->getSelSet(),
           SIGNAL( selectionChanged() ),
           mConfigDlg,
           SLOT( refreshAll() ) );

  connect( mEditor,
           SIGNAL( selectionChanged() ),
           mConfigDlg,
           SLOT( refreshAll() ) );

}

bool GLIMSProject::toXML(
                        QDomDocument &doc,
                        QDomElement &elem,
                        std::string ) {
  ImageState state = mImg->getImageState();
  state.toXML( doc, elem );
  mGData->toXML( doc, elem );
  return true;
}

bool GLIMSProject::fromXML( QDomElement &elem ) {

  // pull version date from the GLIMS_Project root node
  QDomNode rootNode = elem.parentNode() ;
  QDomElement rootElement ;

  if ( !rootNode.isNull() ) {
    rootElement = rootNode.toElement() ;
  } else return false ;


  if ( !rootElement.isNull() && rootElement.hasAttribute ("version") ) {
    stringToQDate((rootElement.attribute("version")).ascii(), mVersion) ;
  }



  QDomNode node;
  if ( !mImgState.fromXML( elem ) )
    return false;

  Image *img = NULL;
  do {
    img = Image::openImage( mImgState.mFileName,
                            (Image::MultiFileType)mImgState.mType );
    if ( !img ) {
      string msg = "Could not open image:\n";
      msg = msg + mFileName;
      int ret = QMessageBox::information( NULL,
                                          "Open Image error",
                                          msg.c_str(),
                                          "Continue",
                                          "Cancel" );
      if ( ret == 1 )
        return false;

      ImageDlg *imgdlg = new ImageDlg( mImgState.mFileName, mImgState.mType );
      imgdlg->exec();
      delete imgdlg;
    }
  } while ( img == NULL );

  img->setBandSel( mImgState );
  setup( img );

  node = elem.nextSibling();
  if ( !node.isElement() )
    return false;
  elem = node.toElement();
  mGData->fromXML( elem );


  // call for updates to project info based on version
  mGData->updateVersion(mVersion) ;

  mConfigDlg->resetAll();
  return true;
}

void GLIMSProject::openConfigDlg( ) {
  mConfigDlg->hide();
  mConfigDlg->show();
  mConfigDlg->setFocus();
}

void GLIMSProject::exportGLIMSIngest( ) {
  DBFHandle           hDBF = NULL;
  SHPHandle           hSHP = NULL;
  double              *vertX, *vertY, *vertZ;
  double              x, y;
  std::string         fname, dirname;
  unsigned int        i;

  setTimeStamp();

  QString qdir = QFileDialog::getExistingDirectory();
  if ( qdir.isNull() )
    return;
  dirname = (const char*)qdir;

  // -------------------
  // SESSION FILE
  fname = dirname + "session";
  Session &session = mGData->getSession();
  hSHP = SHPCreate( fname.c_str(), SHPT_POINTZ );
  if ( hSHP == NULL ) {
    QMessageBox::information( NULL, "File Error", "Can not write Shape Files" );
    return;
  }


  int pdSize = 0 ;
  if (session.mProcDesc.size()) {
    pdSize = session.mProcDesc.size() ;
  }
  else {
    pdSize = 1 ;
  }

  int tdSize = 0 ;
  if (session.m3d_desc.size()) {
    tdSize  = session.m3d_desc.size() ;
  }
  else {
    tdSize = 1 ;
  }


  DBFHeader sesshead[] = {
    { "RC_ID", FTInteger, 11, 0},
    { "data_src", FTString, 255, 0},
    { "proc_desc", FTString, pdSize, 0},
    { "anlst_surn", FTString, 50, 0},
    { "anlst_givn", FTString, 50, 0},
    { "3d_desc", FTString, tdSize, 0},
    { "analy_time", FTString, 35, 0}
  };

  hDBF = DBFCreate( fname.c_str() );
  for ( i=0; i < 7; i++ )
    DBFAddField( hDBF,
                 sesshead[i].fieldName,
                 sesshead[i].type,
                 sesshead[i].width,
                 sesshead[i].nDec );

  DBFWriteIntegerAttribute( hDBF, 0, 0, session.mRC_ID );
  DBFWriteStringAttribute ( hDBF, 0, 1, session.mDataSrc.c_str() );
  DBFWriteStringAttribute ( hDBF, 0, 2, session.mProcDesc.c_str() );
  DBFWriteStringAttribute ( hDBF, 0, 3, session.manlst_surn.c_str() );
  DBFWriteStringAttribute ( hDBF, 0, 4, session.manlst_givn.c_str() );
  DBFWriteStringAttribute ( hDBF, 0, 5, session.m3d_desc.c_str() );
  DBFWriteStringAttribute ( hDBF, 0, 6, session.manaly_time.c_str() );

  vertX = new double[1];
  vertY = new double[1];
  vertZ = new double[1];
  x = mImg->width() / 2;
  y = mImg->height() / 2;
  mImg->getLL( x, y );
  vertX[0] = x;
  vertY[0] = y;
  vertZ[0] = 0;
  SHPObject *shp = SHPCreateSimpleObject( SHPT_POINTZ, 1, vertX, vertY, vertZ );
  delete [] vertX;
  delete [] vertY;
  delete [] vertZ;

  SHPWriteObject( hSHP, -1, shp );
  SHPDestroyObject( shp );

  SHPClose( hSHP );
  DBFClose( hDBF );


  // ----------------------------
  // CREATE GLACIER DATA FILE
  fname = dirname + "glaciers";
  hSHP = SHPCreate( fname.c_str(), SHPT_POINTZ );

  DBFHeader glachead[] = {
    { "ID", FTString, 20, 0},
    { "name", FTString, 40, 0},
    { "prim_class", FTInteger, 5, 0},
    { "form", FTInteger, 5, 0},
    { "front_char", FTInteger, 5, 0},
    { "long_char", FTInteger, 5, 0},
    { "mass_src", FTInteger, 5, 0},
    { "tongue_act", FTInteger, 5, 0},
    { "width_m", FTDouble, 5, 5},
    { "length_m", FTDouble, 5, 5},
    { "area_km2", FTDouble, 5, 5},
    { "abarea_km2", FTDouble, 5, 5},
    { "speed_myr", FTDouble, 5, 5},
    { "snwln_elev", FTDouble, 5, 5},
    { "wgms_id", FTString, 14, 0},
    { "local_id", FTString, 20, 0},
    { "parent_id", FTString, 20, 0},

    // TODO added ad-hoc - need to check correct
    // tag and length values
    { "ela", FTDouble, 5, 5},
    { "ela_desc", FTString, 255, 0},
    { "rec_status", FTString, 255, 0}
  };

  hDBF = DBFCreate( fname.c_str() );
  for ( i=0; i < 20; i++ )
    DBFAddField( hDBF,
                 glachead[i].fieldName,
                 glachead[i].type,
                 glachead[i].width,
                 glachead[i].nDec );

  // get the Glacier Definition Set (Glacier definition points created by the user)
  std::vector<GlacierIDDef> &giddefset = mGData->getGIDData().getDefSet();

  // for each Glacier Definition
  for ( i=0; i < giddefset.size(); i++ ) {
    GlacierIDDef gid = giddefset[i];
    DBFWriteStringAttribute( hDBF, i, 0, gid.toString().c_str() );
    DBFWriteStringAttribute( hDBF, i, 1, gid.mName.c_str() );
    DBFWriteIntegerAttribute( hDBF, i, 2, gid.mPrimCsfn );
    DBFWriteIntegerAttribute( hDBF, i, 3, gid.mForm );
    DBFWriteIntegerAttribute( hDBF, i, 4, gid.mFChar );
    DBFWriteIntegerAttribute( hDBF, i, 5, gid.mLChar );
    DBFWriteIntegerAttribute( hDBF, i, 6, gid.mDomMassSrc );
    DBFWriteIntegerAttribute( hDBF, i, 7, gid.mTongueAct );
    DBFWriteDoubleAttribute( hDBF, i, 8, gid.mWidth );
    DBFWriteDoubleAttribute( hDBF, i, 9, gid.mLength );
    DBFWriteDoubleAttribute( hDBF, i, 10, gid.mArea );
    DBFWriteDoubleAttribute( hDBF, i, 11, gid.mAbzoneArea );
    DBFWriteDoubleAttribute( hDBF, i, 12, gid.mSpeed );
    DBFWriteDoubleAttribute( hDBF, i, 13, gid.mSnowLineElev );
    DBFWriteStringAttribute( hDBF, i, 14, gid.mWGMSID.c_str() );
    DBFWriteStringAttribute( hDBF, i, 15, gid.mLocalID.c_str() );
    DBFWriteStringAttribute( hDBF, i, 16, gid.mParentID.c_str() );

    // TODO added ad hoc - need to check correct tag, len
    DBFWriteDoubleAttribute( hDBF, i, 17, gid.mELA );
    DBFWriteStringAttribute( hDBF, i, 18, gid.mELADesc.c_str() );
    DBFWriteStringAttribute( hDBF, i, 19, gid.mRecordStatus.c_str() );


    vertX = new double[1];
    vertY = new double[1];
    vertZ = new double[1];
    vertX[0] = gid.mLon;
    vertY[0] = gid.mLat;
    vertZ[0] = 0;
    shp = SHPCreateSimpleObject( SHPT_POINTZ, 1, vertX, vertY, vertZ );

    delete [] vertX;
    delete [] vertY;
    delete [] vertZ;

    SHPWriteObject( hSHP, -1, shp );
    SHPDestroyObject( shp );

  }

  SHPClose( hSHP );
  DBFClose( hDBF );


  // ---------------------
  // CREATE IMAGE FILE
  fname = dirname + "images";
  hSHP = SHPCreate( fname.c_str(), SHPT_ARCZ );
  ImageInf &imginf = mGData->getImageInf();

  int imidSize = 1 ;
  if (imginf.mImageID.size() > 0) {
    imidSize = imginf.mImageID.size() ;
  }

  int inidSize = 1 ;
  if (imginf.mInstID.size() > 0) {
    inidSize = imginf.mInstID.size() ;
  }

  DBFHeader imaghead[] = {
    { "image_id", FTString, imidSize, 0},
    { "inst_id", FTString, inidSize, 0},
    { "orig_id", FTString, 50, 0},
    { "imglocurl", FTString, 120, 0},
    { "acq_time", FTString, 35, 0},
    { "imgctrlon", FTDouble, 5, 5},
    { "imgctrlat", FTDouble, 5, 5},
    { "img_lon_unc", FTDouble, 5, 5},
    { "img_lat_unc", FTDouble, 5, 5},
    { "image_azim", FTDouble, 7, 4},
    { "cloud_pct", FTDouble, 3, 9},
    { "sun_azim", FTDouble, 3, 4},
    { "sun_elev", FTDouble, 3, 4},
    { "inst_zen", FTDouble, 3, 4},
    { "inst_azim", FTDouble, 3, 4},
    { "projection", FTString, 30, 0}
  };

  hDBF = DBFCreate( fname.c_str() );
  for ( i = 0; i < 16; i++ ) {
    DBFAddField( hDBF,
                 imaghead[i].fieldName,
                 imaghead[i].type,
                 imaghead[i].width,
                 imaghead[i].nDec );
  }

  DBFWriteStringAttribute( hDBF, 0, 0, imginf.mImageID.c_str() );
  DBFWriteStringAttribute( hDBF, 0, 1, imginf.mInstID.c_str() );
  DBFWriteStringAttribute( hDBF, 0, 2, imginf.mOrigID.c_str() );
  DBFWriteStringAttribute( hDBF, 0, 3, imginf.mLocURL.c_str() );
  DBFWriteStringAttribute( hDBF, 0, 4, imginf.mAquiDateTime.c_str() );
  DBFWriteDoubleAttribute( hDBF, 0, 5, imginf.mCenterLon );
  DBFWriteDoubleAttribute( hDBF, 0, 6, imginf.mCenterLat );
  DBFWriteDoubleAttribute( hDBF, 0, 7, imginf.mCenterLonUnc );
  DBFWriteDoubleAttribute( hDBF, 0, 8, imginf.mCenterLatUnc );
  DBFWriteDoubleAttribute( hDBF, 0, 9, imginf.mImageAzim );
  DBFWriteDoubleAttribute( hDBF, 0, 10, imginf.mCloudPct );
  DBFWriteDoubleAttribute( hDBF, 0, 11, imginf.mSunAzim );
  DBFWriteDoubleAttribute( hDBF, 0, 12, imginf.mSunElev );
  DBFWriteDoubleAttribute( hDBF, 0, 13, imginf.mInstrument_zenith );
  DBFWriteDoubleAttribute( hDBF, 0, 14, imginf.mInstrument_azimuth );
  DBFWriteStringAttribute( hDBF, 0, 15, imginf.mProjection.c_str() );


  vertX = new double[5];
  vertY = new double[5];
  vertZ = new double[5];
  mImg->setCornerPnts( imginf );
  vertX[0] = imginf.mP1.x;  // PNT 1
  vertY[0] = imginf.mP1.y;
  vertZ[0] = 0;
  vertX[1] = imginf.mP2.x;  // PNT 2
  vertY[1] = imginf.mP2.y;
  vertZ[1] = 0;
  vertX[2] = imginf.mP3.x;  // PNT 3
  vertY[2] = imginf.mP3.y;
  vertZ[2] = 0;
  vertX[3] = imginf.mP4.x;  // PNT 4
  vertY[3] = imginf.mP4.y;
  vertZ[3] = 0;
  vertX[4] = imginf.mP1.x;  // PNT 5
  vertY[4] = imginf.mP1.y;
  vertZ[4] = 0;
  shp = SHPCreateSimpleObject( SHPT_ARCZ, 5, vertX, vertY, vertZ );
  delete [] vertX;
  delete [] vertY;
  delete [] vertZ;

  SHPWriteObject( hSHP, -1, shp );
  SHPDestroyObject( shp );

  SHPClose( hSHP );
  DBFClose( hDBF );



  // -----------------------
  // CREATE SEGMENTS DBF FILE

  fname = dirname + "segments";
  hSHP = SHPCreate( fname.c_str(), SHPT_ARCZ );
  DBFHeader seghead[] = {
    { "category",   FTString, 20, 0},
    { "ID",         FTString, 20, 0},
    { "type",       FTString, 30, 0},
    { "label",      FTString, 32, 0},
    { "loc_unc_x",  FTDouble, 5,  5},
    { "loc_unc_y",  FTDouble, 5,  5},
    { "glob_unc_x", FTDouble, 5,  5},
    { "glob_unc_y", FTDouble, 5,  5},
    { "left_mat",   FTString, 3,  0},
    { "right_mat",  FTString, 3,  0},
    { "left_feat",  FTString, 3,  0},
    { "right_feat", FTString, 3,  0},
    { "ortho",      FTString, 1,  0}
  };

  hDBF = DBFCreate( fname.c_str() );
  for ( i=0; i < 13; i++ ) {
    DBFAddField( hDBF,
                 seghead[i].fieldName,
                 seghead[i].type,
                 seghead[i].width,
                 seghead[i].nDec );
  }

  std::vector<Shape*> &lineset = mGData->getLineData().getLLSet();
  LineDefSet &ldfset = mGData->getLineData().getLDFSet();
  std::vector<GlacierIDDef> &gidset = mGData->getGIDData().getDefSet();

  for ( i=0; i < lineset.size(); i++ ) {
    char buf[50];
    std::string gidlbl;

    Shape &s = *lineset[i];
    GlacierLine &gline = dynamic_cast<GlacierLine&>( s );
    LineDef ldf;

    if ( gline.mLDF >= 0 ) {
      ldf = ldfset[gline.mLDF];
    }

    sprintf( buf,
             "gtype: %d, lat: %d, label: %d",
             ldf.mGlacType, ldf.mLMat, ldf.mLabel );

    gidlbl = "UNDEFINED";
    if ( gline.mGID >= 0 && gline.mGID < (int)gidset.size() ) {
      gidlbl = gidset[gline.mGID].toString();
    }

    DBFWriteStringAttribute( hDBF, i, 0, GLIMSGlobals::mGlbGlacTypes[ldf.mGlacType].id.c_str() );
    DBFWriteStringAttribute( hDBF, i, 1, gidlbl.c_str() );
    DBFWriteStringAttribute( hDBF, i, 2, GLIMSGlobals::mGlbTypes[ldf.mType].id.c_str() );
    DBFWriteStringAttribute( hDBF, i, 3, GLIMSGlobals::mGlbLbls[ldf.mLabel].id.c_str() );
    DBFWriteDoubleAttribute( hDBF, i, 4, ldf.mLXUncert );
    DBFWriteDoubleAttribute( hDBF, i, 5, ldf.mLYUncert );
    DBFWriteDoubleAttribute( hDBF, i, 6, ldf.mGXUncert );
    DBFWriteDoubleAttribute( hDBF, i, 7, ldf.mGYUncert );
    DBFWriteStringAttribute( hDBF, i, 8, GLIMSGlobals::mGlbMDF[ldf.mLMat].id.c_str() );
    DBFWriteStringAttribute( hDBF, i, 9, GLIMSGlobals::mGlbMDF[ldf.mRMat].id.c_str() );
    DBFWriteStringAttribute( hDBF, i, 10, GLIMSGlobals::mGlbFtrs[ldf.mLFeature].id.c_str() );
    DBFWriteStringAttribute( hDBF, i, 11, GLIMSGlobals::mGlbFtrs[ldf.mRFeature].id.c_str() );
    //DBFWriteStringAttribute( hDBF, i, 12, GLIMSGlobals::mGlbFtrs[ldf.mRFeature].id.c_str() );


    int npts = gline.size();
    vertX = new double[ npts ];
    vertY = new double[ npts ];
    vertZ = new double[ npts ];
    for ( int a=0; a < npts; a++ ) {
      vertX[a] = gline[a].x;
      vertY[a] = gline[a].y;
      vertZ[a] = gline[a].z;
    }

    shp = SHPCreateSimpleObject( SHPT_ARCZ, npts, vertX, vertY, vertZ );
    delete [] vertX;
    delete [] vertY;
    delete [] vertZ;

    SHPWriteObject( hSHP, -1, shp );
    SHPDestroyObject( shp );
  }

  SHPClose( hSHP );
  DBFClose( hDBF );

  mConfigDlg->resetAll() ;

  return;
}

void GLIMSProject::importFromShapefile( ) {
  int nrec = 0;
  std::string fname;

  QString qfname = QFileDialog::getOpenFileName();
  if ( qfname.isNull() )
    return;
  fname = (const char*)qfname;

  // LOAD THE SEGMENTS SHAPE FILE( YARG )
  SHPHandle hshp = SHPOpen( fname.c_str(), "rb" );
  if ( !hshp )
    return;

  SHPGetInfo( hshp, &nrec, NULL, NULL, NULL );

  for ( int ildf=0; ildf < nrec; ildf++ ) {
    SHPObject *shpobj = SHPReadObject( hshp, ildf );
    if ( !shpobj || shpobj->nVertices <= 0 )
      continue;

    Node n;
    n.x = shpobj->padfX[0];
    n.y = shpobj->padfY[0];
    n.z = shpobj->padfZ[0];
    mGData->getLineData().addLineLL( n );
    int iline = mGData->getLineData().size()-1;
    for ( int inode=1; inode < shpobj->nVertices; inode++ ) {
      Node n;
      n.x = shpobj->padfX[inode];
      n.y = shpobj->padfY[inode];
      n.z = shpobj->padfZ[inode];
      mGData->getLineData().addLineNodeLL( n, iline );
    }
    // mGData->getLineData().setGID( iline, -1 );
    // mGData->getLineData().setLDF( iline, -1 );
  }
}

void GLIMSProject::importGLIMSIngest( ) {
  std::string     prefix = "";
  std::string     dir;
  std::string     fname;
  DBFHandle       hdbf;
  SHPHandle       hshp;
  SHPObject       *shpobj;
  int             pos;
  int             nrec;
  ImageInf        imginf;

  const int NSHPFILES = 4;
  const std::string shp_names[] = {
    "images",
    "glaciers",
    "segments",
    "session"
  };

  // GET ONE OF THE SHAPEFILES OF THE SET
  QString dirstr = QFileDialog::getOpenFileName();

  if ( dirstr.isNull() ) {
    mOpen = false;
    return;
  }


  // GET THE PREFIX OF THE SHAPEFILE( AKA PROJECT NAME )
  prefix = (const char*)dirstr;
  //!@internal Bad cross-platform here, but not finding better (note by Dan Mingus)
  //!@todo look at what's being done here and why, and possibly change to using a Qt
  //!      object to handle the situation (note by deborah)
  pos = prefix.find_last_of( '\\' );
  if ( pos == -1 ) {
    pos = prefix.find_last_of( '/' );
  }
  dir = prefix.substr( 0, pos + 1 );

  prefix = prefix.substr( pos + 1, prefix.length() - pos );
  for ( int i=0; i < NSHPFILES; i++ ) {
    if ( (pos = prefix.find( shp_names[i] )) != -1 ) {
      prefix = prefix.substr( 0, pos );
      break;
    }
  }

  // CHECK SHAPFILE SET OUT.  IF THIS RETURNS TRUE, THEN
  // THERE'S NO REASON TO TEST THE SHAPEFILES FOR BEING OPEN
  // OR COMPLETE LATER.
  //!@todo 2006-01-21 dls Considering taking this check out completely
  if ( !checkShapefileSet( dir + prefix ) ) {
    QString msgdir = dir.c_str() ;
    QString msgprf = prefix.c_str() ;
    QString msg  = "Invalid shapefile dataset\n" + msgdir + msgprf ;
    QMessageBox::information( NULL,
                              "Ingest Import Error",
                              msg );
    mOpen = false;
    return;
  }

  //-------------------------------------
  // LOAD THE IMAGES SHAPE FILE
  fname = dir + "images.shp";
  hdbf = DBFOpen( fname.c_str(), "rb" );

  imginf.mImageID = DBFReadStringAttribute( hdbf, 0, 0 );
  imginf.mInstID  = DBFReadStringAttribute( hdbf, 0, 1 );
  imginf.mOrigID  = DBFReadStringAttribute( hdbf, 0, 2 );
  imginf.mLocURL  = DBFReadStringAttribute( hdbf, 0, 3 );
  imginf.mAquiDateTime = DBFReadStringAttribute( hdbf, 0, 4 );
  imginf.mCenterLon = DBFReadDoubleAttribute( hdbf, 0, 5 );
  imginf.mCenterLat = DBFReadDoubleAttribute( hdbf, 0, 6 );
  imginf.mCenterLonUnc = DBFReadDoubleAttribute( hdbf, 0, 7 );
  imginf.mCenterLatUnc = DBFReadDoubleAttribute( hdbf, 0, 8 );
  imginf.mImageAzim = DBFReadDoubleAttribute( hdbf, 0, 9 );
  imginf.mCloudPct = DBFReadDoubleAttribute( hdbf, 0, 10 );
  imginf.mSunAzim = DBFReadDoubleAttribute( hdbf, 0, 11 );
  imginf.mSunElev = DBFReadDoubleAttribute( hdbf, 0, 12 );
  imginf.mInstrument_zenith = DBFReadDoubleAttribute( hdbf, 0, 13 );
  imginf.mInstrument_azimuth = DBFReadDoubleAttribute( hdbf, 0, 14 );
  imginf.mProjection = DBFReadStringAttribute( hdbf, 0, 15 );

  DBFClose( hdbf );

  Image *img = Image::openImage( imginf.mLocURL.c_str() );
  while ( img == NULL ) {
    std::string imgloc;
    int type;
    QMessageBox::information( NULL,
                              "Open Image Error",
                              "Unable to locate image.  Please specify" );

    ImageDlg *imgdlg = new ImageDlg( imgloc, type );
    if ( !imgdlg->exec() ) {
      mOpen = false;
      delete imgdlg;
      return;
    }
    delete imgdlg;
    img = Image::openImage( imgloc, (Image::MultiFileType)type );
  }

  setup( img );
  mGData->getImageInf() = imginf;

  //-------------------------------------
  // LOAD THE SESSION SHAPE FILE
  fname = dir + "session.shp";
  hdbf = DBFOpen( fname.c_str(), "rb" );
  Session &ses = mGData->getSession();

  ses.mRC_ID =        DBFReadIntegerAttribute( hdbf, 0, 0 );
  ses.mDataSrc =      DBFReadStringAttribute( hdbf, 0, 1 );
  ses.mProcDesc =     DBFReadStringAttribute( hdbf, 0, 2 );
  ses.manlst_surn =   DBFReadStringAttribute( hdbf, 0, 3 );
  ses.manlst_givn =   DBFReadStringAttribute( hdbf, 0, 4 );
  ses.m3d_desc =      DBFReadStringAttribute( hdbf, 0, 5 );
  ses.manaly_time =   DBFReadStringAttribute( hdbf, 0, 6 );

  DBFClose( hdbf );


  //-------------------------------------
  // LOAD THE GLACIERS SHAPE FILE
  fname = dir + "glaciers.shp";
  hshp = SHPOpen( fname.c_str(), "rb" );
  if ( !hshp ) return;
  hdbf = DBFOpen( fname.c_str(), "rb" );

  nrec = DBFGetRecordCount( hdbf );
  for ( int igid=0; igid < nrec; igid++ ) {
    Node n;
    GlacierIDDef gid;
    gid.mName = DBFReadStringAttribute( hdbf, igid, 1 );
    gid.mPrimCsfn = DBFReadIntegerAttribute( hdbf, igid, 2 );
    gid.mForm = DBFReadIntegerAttribute( hdbf, igid, 3);
    gid.mFChar = DBFReadIntegerAttribute( hdbf, igid, 4 );
    gid.mLChar = DBFReadIntegerAttribute( hdbf, igid, 5 );
    gid.mDomMassSrc = DBFReadIntegerAttribute( hdbf, igid, 6 );
    gid.mTongueAct = DBFReadIntegerAttribute( hdbf, igid, 7 );
    gid.mWidth = DBFReadDoubleAttribute( hdbf, igid, 8 );
    gid.mLength = DBFReadDoubleAttribute( hdbf, igid, 9 );
    gid.mArea = DBFReadDoubleAttribute( hdbf, igid, 10 );
    gid.mAbzoneArea = DBFReadDoubleAttribute( hdbf, igid, 11 );
    gid.mSpeed = DBFReadDoubleAttribute( hdbf, igid, 12 );
    gid.mSnowLineElev = DBFReadDoubleAttribute( hdbf, igid, 13 );
    gid.mWGMSID = DBFReadStringAttribute( hdbf, igid, 14 );
    gid.mLocalID = DBFReadStringAttribute( hdbf, igid, 15 );
    gid.mParentID = DBFReadStringAttribute( hdbf, igid, 16 );

    // dls 2006-01-07 added grabbing ela, rec status
    gid.mELA          = DBFReadDoubleAttribute( hdbf, igid, 17 );
    gid.mELADesc      = DBFReadStringAttribute( hdbf, igid, 18 );
    gid.mRecordStatus = DBFReadStringAttribute( hdbf, igid, 19 );

    shpobj = SHPReadObject( hshp, igid );
    if ( shpobj->nVertices < 1 )
      continue;

    gid.mLon = shpobj->padfX[0];
    gid.mLat = shpobj->padfY[0];
    // gid.mHeight = shpobj->padfZ[0]; // <---- NOT SUPPORTING ELEVATION YET

    SHPDestroyObject( shpobj );

    mGData->getGIDData().addGID( gid );

  }


  //-------------------------------------
  // LOAD THE SEGMENTS SHAPE FILE( YARG )
  fname = dir + "segments.shp";
  hshp = SHPOpen( fname.c_str(), "rb" );
  if ( !hshp )
    return;
  hdbf = DBFOpen( fname.c_str(), "rb" );
  nrec = DBFGetRecordCount( hdbf );
  std::string gidid, glactype, type, label;
  std::string lmat, rmat, lfeat, rfeat, ortho;
  int stylecnt = 0;
  for ( int ildf=0; ildf < nrec; ildf++ ) {
    int indgid=0, indldf=0;
    LineDef ldf, DEFAULT_LDF;
    glactype = DBFReadStringAttribute( hdbf, ildf, 0 );
    gidid = DBFReadStringAttribute( hdbf, ildf, 1 );
    type = DBFReadStringAttribute( hdbf, ildf, 2 );
    label = DBFReadStringAttribute( hdbf, ildf, 3 );
    ldf.mLXUncert = DBFReadDoubleAttribute( hdbf, ildf, 4 );
    ldf.mLYUncert = DBFReadDoubleAttribute( hdbf, ildf, 5 );
    ldf.mGXUncert = DBFReadDoubleAttribute( hdbf, ildf, 6 );
    ldf.mGYUncert = DBFReadDoubleAttribute( hdbf, ildf, 7 );
    lmat = DBFReadStringAttribute( hdbf, ildf, 8 );
    rmat = DBFReadStringAttribute( hdbf, ildf, 9 );
    lfeat = DBFReadStringAttribute( hdbf, ildf, 10 );
    rfeat = DBFReadStringAttribute( hdbf, ildf, 11 );
    ortho = DBFReadStringAttribute( hdbf, ildf, 12 );

    ldf.mGlacType = findIndex( glactype, GLIMSGlobals::mGlbGlacTypes );
    ldf.mType = findIndex( type, GLIMSGlobals::mGlbTypes );
    ldf.mLabel = findIndex( label, GLIMSGlobals::mGlbLbls );
    ldf.mLMat = findIndex( lmat, GLIMSGlobals::mGlbMDF );
    ldf.mRMat = findIndex( rmat, GLIMSGlobals::mGlbMDF );
    ldf.mLFeature = findIndex( lfeat, GLIMSGlobals::mGlbFtrs );
    ldf.mRFeature = findIndex( rfeat, GLIMSGlobals::mGlbFtrs );

    LineDefSet &defset = mGData->getLineData().getLDFSet();
    int i;
    for ( i=0; i < (int)defset.size(); i++ ) {
      if ( defset[i] == ldf ) {
        indldf = i;
        break;
      }
    }

    if ( i >= (int)defset.size() && ldf != DEFAULT_LDF ) {
      ldf.mColor = stylecnt % 8;
      ldf.mWidth = 1+ ( (stylecnt%24) / 8);
      ldf.mStyle = 1+ ( (stylecnt%72) / 24);
      mGData->getLineData().addLDF( ldf );
      indldf = i;
      stylecnt++;
    }

    std::vector<GlacierIDDef> &giddefset =
    mGData->getGIDData().getDefSet();
    for ( i=0; i < (int)giddefset.size(); i++ ) {
      if ( gidid == giddefset[i].toString() )
        indgid = i;
    }

    shpobj = SHPReadObject( hshp, ildf );
    if ( !shpobj || shpobj->nVertices <= 0 )
      continue;

    Node n;
    n.x = shpobj->padfX[0];
    n.y = shpobj->padfY[0];
    n.z = shpobj->padfZ[0];
    mGData->getLineData().addLineLL( n );
    int iline = mGData->getLineData().size()-1;
    for ( int inode=1; inode < shpobj->nVertices; inode++ ) {
      Node n;
      n.x = shpobj->padfX[inode];
      n.y = shpobj->padfY[inode];
      n.z = shpobj->padfZ[inode];
      mGData->getLineData().addLineNodeLL( n, iline );
    }
    mGData->getLineData().setGID( iline, indgid );
    mGData->getLineData().setLDF( iline, indldf );
  }

  mOpen = true;

}

void GLIMSProject::setTimeStamp( ) {
  char buf[2560];
  struct tm *t;
  const time_t local_time = time( NULL );

  t = gmtime( &local_time );
  if ( t == NULL )
    return;

  sprintf( buf,
           "%d%02d%02dT%02d%02d%02d-0000",
           1900+t->tm_year,
           1+t->tm_mon,
           t->tm_mday,
           t->tm_hour,
           t->tm_min,
           t->tm_sec );

  mGData->getSession().manaly_time = buf;
}

/*
DBFHandle GLIMSProject::createDBFH( const char* fname,
                                    DBFHeader *head,
                                    int nent )
{
    DBFHandle hDBF = DBFCreate( fname );
    for( int i=0; i < nent; i++ )
        DBFAddField( hDBF,
                     head[i].fieldName,
                     head[i].type,
                     head[i].width,
                     head[i].nDec );

    return hDBF;
}
*/

bool GLIMSProject::checkShapefileSet( const std::string &prefix ) const {
  DBFHandle       hdbf;
  DBFFieldType    type;
  char            *name = new char[11];
  std::string     test1, test2;
  int             width;
  int             ndec;
  int             ncol;
  int             icol;
  // TEST ****IMAGES.SHP

  DBFHeader imghead[] = {
    { "image_id", FTString, 10, 0},
    { "inst_id", FTString, 10, 0},
    { "orig_id", FTString, 50, 0},
    { "imglocurl", FTString, 120, 0},
    { "acq_time", FTString, 35, 0},
    { "imgctrlon", FTDouble, 5, 5},
    { "imgctrlat", FTDouble, 5, 5},
    { "img_lon_unc", FTDouble, 5, 5},
    { "img_lat_unc", FTDouble, 5, 5},
    { "image_azim", FTDouble, 7, 4},
    { "cloud_pct", FTDouble, 3, 9},
    { "sun_azim", FTDouble, 3, 4},
    { "sun_elev", FTDouble, 3, 4},
    { "inst_zen", FTDouble, 3, 4},
    { "inst_azim", FTDouble, 3, 4},
    { "projection", FTString, 30, 0}
  };
  ncol = 16; // THE NUMBER OF DBF COLUMNS ABOVE

  // open file, fail if it doesn't exist
  hdbf = DBFOpen( (prefix + "images.shp").c_str(), "rb" );
  if ( !hdbf ) {
    return false;
  }

  // check data, fail if it's not correct
  for ( icol=0; icol < ncol; icol++ ) {
    type = DBFGetFieldInfo( hdbf, icol, name, &width, &ndec );
    test1 = name;
    test2 = imghead[icol].fieldName;
    if ( test1.length() > 10 )
      test1 = test1.substr( 0, 10 );
    if ( test2.length() > 10 )
      test2 = test2.substr( 0, 10 );

    if ( type != imghead[icol].type ||
         test2 != test1 ||
         // 2006-01-21 dls Took out width check, some fields are not limited in size
         //width != imghead[icol].width ||
         ndec != imghead[icol].nDec ) {
      return false;
    }
  }
  DBFClose( hdbf );

  // TEST ****SESSION.SHP
  DBFHeader sesshead[] = {
    { "RC_ID", FTInteger, 11, 0},
    { "data_src", FTString, 255, 0},
    { "proc_desc", FTString, 200, 0},
    { "anlst_surn", FTString, 50, 0},
    { "anlst_givn", FTString, 50, 0},
    { "3d_desc", FTString, 200, 0},
    { "analy_time", FTString, 35, 0}
  };
  ncol = 7;

  // open file, fail if it doesn't exist
  hdbf = DBFOpen( (prefix + "session.shp").c_str(), "rb" );
  if ( !hdbf ) {
    return false;
  }

  // check data, fail if it's not correct
  for ( icol=0; icol < ncol; icol++ ) {
    type = DBFGetFieldInfo( hdbf, icol, name, &width, &ndec );
    test1 = name;
    test2 = sesshead[icol].fieldName;
    if ( test1.length() > 10 )
      test1 = test1.substr( 0, 10 );
    if ( test2.length() > 10 )
      test2 = test2.substr( 0, 10 );

    if ( type != sesshead[icol].type ||
         test2 != test1 ||
         // dls 2006-01-06 -- removed this check, some fields are not limited in size
         //width != sesshead[icol].width ||
         ndec != sesshead[icol].nDec ) {
      return false;
    }
  }
  DBFClose( hdbf );

  // TEST ****GLACIERS.SHP
  DBFHeader glachead[] = {
    { "ID", FTString, 20, 0},
    { "name", FTString, 40, 0},
    { "primclass", FTInteger, 5, 0},
    { "form", FTInteger, 5, 0},
    { "front_char", FTInteger, 5, 0},
    { "long_char", FTInteger, 5, 0},
    { "mass_src", FTInteger, 5, 0},
    { "tongue_act", FTInteger, 5, 0},
    { "width_m", FTDouble, 5, 5},
    { "length_m", FTDouble, 5, 5},
    { "area_km2", FTDouble, 5, 5},
    { "abarea_km2", FTDouble, 5, 5},
    { "speed_myr", FTDouble, 5, 5},
    { "snwln_elev", FTDouble, 5, 5},
    { "wgms_id", FTString, 14, 0},
    { "local_id", FTString, 20, 0},
    { "parent_id", FTString, 20, 0}
  };
  ncol = 17;

  bool checkStatus = true ;
  hdbf = DBFOpen( (prefix + "glaciers.shp").c_str(), "rb" );
  if ( !hdbf ) {
    return false;
  }
  for ( icol=0; icol < ncol; icol++ ) {
    type = DBFGetFieldInfo( hdbf, icol, name, &width, &ndec );
    test1 = name;
    test2 = glachead[icol].fieldName;
    if ( test1.length() > 10 )
      test1 = test1.substr( 0, 10 );
    if ( test2.length() > 10 )
      test2 = test2.substr( 0, 10 );

    if ( type != glachead[icol].type ||
         test2 != test1 ||
         // 2006-01-21 dls Took out width check, fields may not be limited in size
         //width != glachead[icol].width ||
         ndec != glachead[icol].nDec ) {
      checkStatus = false;
    }
  }

  // check for older version of shapefile
  if ( checkStatus == false ) {
    // TEST ****GLACIERS.SHP
    DBFHeader glacheadRetry[] = {
      { "ID", FTString, 20, 0},
      { "name", FTString, 40, 0},
      { "prim_class", FTInteger, 5, 0},
      { "form", FTInteger, 5, 0},
      { "front_char", FTInteger, 5, 0},
      { "long_char", FTInteger, 5, 0},
      { "mass_src", FTInteger, 5, 0},
      { "tongue_act", FTInteger, 5, 0},
      { "width_m", FTDouble, 5, 5},
      { "length_m", FTDouble, 5, 5},
      { "area_km2", FTDouble, 5, 5},
      { "abarea_km2", FTDouble, 5, 5},
      { "speed_myr", FTDouble, 5, 5},
      { "snwln_elev", FTDouble, 5, 5},
      { "wgms_id", FTString, 14, 0},
      { "local_id", FTString, 20, 0},
      { "parent_id", FTString, 20, 0}
    };
    ncol = 17;


    for ( icol=0; icol < ncol; icol++ ) {
      type = DBFGetFieldInfo( hdbf, icol, name, &width, &ndec );
      test1 = name;
      test2 = glacheadRetry[icol].fieldName;
      if ( test1.length() > 10 )
        test1 = test1.substr( 0, 10 );
      if ( test2.length() > 10 )
        test2 = test2.substr( 0, 10 );

      if ( type != glacheadRetry[icol].type ||
           test2 != test1 ||
           width != glacheadRetry[icol].width ||
           ndec != glacheadRetry[icol].nDec )  {
      return false;
    }
    }

  }

  DBFClose( hdbf );

  // TEST ****SEGMENTS.SHP
  DBFHeader seghead[] = {
    { "category",   FTString, 20, 0},
    { "ID",         FTString, 20, 0},
    { "type",       FTString, 30, 0},
    { "label",      FTString, 32, 0},
    { "loc_unc_x",  FTDouble, 5,  5},
    { "loc_unc_y",  FTDouble, 5,  5},
    { "glob_unc_x", FTDouble, 5,  5},
    { "glob_unc_y", FTDouble, 5,  5},
    { "left_mat",   FTString, 3,  0},
    { "right_mat",  FTString, 3,  0},
    { "left_feat",  FTString, 3,  0},
    { "right_feat", FTString, 3,  0},
    { "ortho",      FTString, 1,  0}
  };
  ncol = 12;

  hdbf = DBFOpen( (prefix + "segments.shp").c_str(), "rb" );
  if ( !hdbf ) {
    return false;
  }
  for ( icol=0; icol < ncol; icol++ ) {
    type = DBFGetFieldInfo( hdbf, icol, name, &width, &ndec );
    test1 = name;
    test2 = seghead[icol].fieldName;
    if ( test1.length() > 10 )
      test1 = test1.substr( 0, 10 );
    if ( test2.length() > 10 )
      test2 = test2.substr( 0, 10 );

    if ( type != seghead[icol].type ||
         test2 != test1 ||
         // 2006-01-21 dls Took out width check, fields may not be limited in size
         //width != seghead[icol].width ||
         ndec != seghead[icol].nDec ) {
      return false;
    }
  }
  DBFClose( hdbf );

  delete [] name;
  // MADE IT ALL SEEMS GOOD
  return true;
}

bool GLIMSProject::close( ) {

  if ( mNeedSave ) {
    int ret;
    ret = QMessageBox::information( NULL,
                                    "Project Not Saved",
                                    "The project is not saved.  Would you like to save it?",
                                    "Yes",
                                    "No",
                                    "Cancel" );
    if ( ret == 0 ) {
      save();
    }

    if ( ret == 2 ) {
      return false ;
    }
  }


  //if( mImg != NULL )
  if (mImg) {
    delete mImg;
  }
  if ( mConfigDlg ) {
    delete mConfigDlg;
  }
  if ( mEditor ) {
    delete mEditor;
  }
  if ( mLyrSet ) {
    delete mLyrSet;
  }
  if ( mGData ) {
    delete mGData;
  }



  mNeedSave = false;
  return true ;

}






