#ifndef __GLIMSPROJECT_H
#define __GLIMSPROJECT_H

#include "windows_defs.h"
#include "configdlg.h"
#include "viewset.h"
#include "image.h"
#include "vectorlayerset.h"
#include "vectoreditor.h"
#include "glimsdataset.h"
#include "selectionset.h"
#include "xmlserializeable.h"
#include "projectdlg.h"
#include "imagedlg.h"

#include <qobject.h>
#include <qwidget.h>
#include <qfiledialog.h>

#include <ctime>

#include <string>



/**
 * This binds the GLIMSDataset to the views, layers, image, and
 * the various dialogs.  Creation of a GLIMSProject results in
 * an initialized GLIMSDataset and ties menu actions pertaining to
 * the dialogs.  This can be saved or opened from it's XML serialization
 * routines or a new project can be created.
 *
 * This also provides routines that import/export a shapefile set that
 * corresponds to a GLIMS ingest.
 *
 * @history 2004-12-04 Deborah Lee Soltesz
 *
 * @history 2005-08-04 Deborah Lee Soltesz - tweaked "saved" status,
 *          exit dialog ("cancel" option to not close app, now returns
 *          a boolean designating whether or not to go ahead with exit)
 *
 *
 */
class GV_EXPORT GLIMSProject : public QObject, public XMLSerializeable {
  Q_OBJECT
  protected:
  Image               *mImg;
  ImageState          mImgState;
  GLIMSDataset        *mGData;
  VectorLayerSet      *mLyrSet;
  VectorEditor        *mEditor;
  ViewSet             *mViewSet;
  ConfigDlg           *mConfigDlg;
  bool                mOpen;
  bool                mNeedSave;
  QWidget             *mParent;
  QDate               *mVersion;

  std::string         mFileName;

  void setup( Image * );

  /**
   * Checks a GLIMS Shapefile set to see if it's valid:
   * (1) whether all files exists
   * (2) whether the files contain the correct data fields
   *
   * @history 2006-01-07 Deborah Lee Soltesz -
   *   Removed "width" check on data fields in session.shp
   *   because some of the fields are unlimited as defined by
   *   the GLIMS database schema.
   */
  bool checkShapefileSet( const std::string &prefix ) const;

  int findIndex( const std::string &str,
                 const std::vector<Attr> &attr ) {
    int i=0;
    for ( ; i < (int)attr.size(); i++ ) {
      if ( attr[i].id == str )
        break;
    }
    if ( i >= (int)attr.size() )
      i = 0;
    return i;
  }

public:
  GLIMSProject( QWidget *, ViewSet * );
  virtual ~GLIMSProject( ) {
    close();
  }

  /**
   * Writes project metadata to the XML-formatted project file
   */
  virtual bool toXML(
                    QDomDocument &doc,
                    QDomElement &elem,
                    std::string id = "" );

  /**
   * Reads project metadata from the XML-formatted project file
   */
  virtual bool fromXML( QDomElement &elem );

  /**
   * Returns a pointer to the Image object, which contains
   * the base image and related image information for the project
   */
  Image*          getImage( ) {
    return mImg;
  }
  /**
   * Returns a pointer to the GLIMSDataset object, which contains
   * all the structures holding metadata, user-digitized data, the base
   * image, and session information
   */
  GLIMSDataset*   getDataset( ) {
    return mGData;
  }
  /**
   * Returns a pointer to the Vector layer
   */
  VectorLayerSet* getVectorLayerSet( ) {
    return mLyrSet;
  }
  /**
   * Returns a pointer to the VectorEditor
   */
  VectorEditor*   getVectorEditor( ) {
    return mEditor;
  }
  /**
   * Returns a pointer to the ViewSet, i.e. the windows showing
   * the image and digitized vector data at various zoom levels and
   * information about selection coordinates
   */
  ViewSet*        getViewSet( ) {
    return mViewSet;
  }
  /**
   * Returns the GLIMSView XML version, which is stored as a date.
   * The GLIMSView XML version is used to "correct" XML changes
   * when functions related to reading/writing the XML project
   * file are modified to the point of actually altering how
   * the XML is written and what values/data are stored.
   */
  QDate*          getVersion( ) {
    return mVersion;
  }
  /**
   * Is there a project open?
   */
  bool            isOpen( ) {
    return mOpen;
  }
  /**
   * Creates a timestamp when the project is exported to
   * GLIMS Shapefile format. The timestamp is reflected in
   * the project as the "analysis time"
   */
  void            setTimeStamp( );

  /**
   * Import vector data from a generic shapefile
   */
  void            importFromShapefile( );


  /**
   * Return the filename of the project
   */
  std::string     getFileName () {
    return mFileName ;
  }

public slots:
  /**
   * Open a project
   *
   * @todo 2005-08-04 deborah lee soltesz
   *       Add filter to file chooser box
   */
  bool open( );
  /**
   * Create a project
   */
  bool create( );
  /**
   * Save the current project
   *
   * @todo 2005-08-04 deborah lee soltesz
   *       Add filter to file chooser box
   */
  bool save( );
  /**
   * Save the current work to a new project file
   * @todo 2005-08-04 deborah lee soltesz
   *       Add filter to file chooser box
   */
  bool saveAs( );
  /**
   * Show the Configuration dialog, which contains the panels
   * for displaying and modifying Image, Session, Line, and Glacier ID
   * information
   */
  void openConfigDlg( );

  /**
   * Export the current project to GLIMS Shapefile format. This
   * format is made up of four shapefiles that contain the Image,
   * Session, GlacierID, and Line Segment data and information.
   * This format is the "final product" created by the analyst
   * for submission to the GLIMS project. The information in the
   * shapefiles is ingested into a geospatial database.
   *
   * @history 2005-08-20 Deborah Lee Soltesz
   *          Fixed crashing probabably caused by trying to create a zero
   *          length string when the procDesc and 3dDesc fields are empty
   *
   * @history 2005-08-04 Deborah Lee Soltesz
   *          Modified to calculate the length of free-text
   *          descriptions for generating shapefile string
   *          (was hard-coded to 200, truncating the fields)
   *
   * @history 2006-01-07 Deborah Lee Soltesz -
   *          Added writing ELA, ELA Description, and Record Status to export
   *
   *
   * @todo 2005-08-04 deborah lee soltesz
   *       Does export handle all the fields correctly,
   *       particularly not truncate text fields?
   *       Add filter to file chooser box
   *
   * @todo 2006-01-07 deborah lee soltesz
   *       Making additions to save ELA, ELA desc, record status
   *       to exported glaciers file - need to make sure tags are
   *       correct (according to GLIMS spec) and are imported
   *       via importGLIMSIngest
   *
   * @history 2006-01-22 Deborah Lee Soltesz -
   *       Added call to refresh configuration dialog to force session
   *       analysis time to update on export (analysis time is considered
   *       to be the time when the project is exported because the export
   *       files are the "final product")
   */
  void exportGLIMSIngest( );

  /**
   * @todo 2005-08-04 deborah lee soltesz
   *       Add filter to file chooser box
   * @todo 2006-01-07 deborah lee soltesz
   *       Making additions to save ELA, ELA desc, record status
   *       to exported glaciers file - need to make sure tags are
   *       correct (according to GLIMS spec) and are imported
   *       correctly in this function
   * @history 2006-01-07 Deborah Lee Soltesz -
   *       Added reading ELA, ELA Description, and Record Status from export
   */
  void importGLIMSIngest( );


  /**
   * Sets status of project to "not saved." Connected
   * to datasetChanged() signals from other classes.
   */
  void setUnsaved( ) {
    mNeedSave = true;
  }

  /**
   * Close the project
   */
  bool close( );

};

#endif



