#ifndef __HDFEOSFRMT_H
#define __HDFEOSFRMT_H

#include "imageformat.h"
#include "odlparser.h"

#include <hdf.h>
#include <mfhdf.h>
#include <HdfEosDef.h>
#include <proj_api.h>

#include <string>
#include <cstring>
#include <vector>
#include <cmath>

#ifndef M_PI
#define M_PI 3.1415926
#endif

/**
 * An ImageFormat implementation providing access to the
 * HdfEos library.  Being very difficult to make a generic
 * implementation that's not specific to a single dataset,
 * much of this code is specific to Aster L1A and L1B products.
 *
 * @history 2006-01-09 dls -
 *          Added several new functions to pull metadata out of
 *          Aster HDF file
 */
class HdfEosFrmt : public ImageFormat {
protected:
  int32   mSWfid;

  /**
   * Aliases for projection names
   */
  typedef enum {
    UNSUPPORTED, //!< unsupported
    UTM          //!< Universal Transverse Mercator
  } Projection ;

/**
 * Structure to hold geographic metadata about the image
 */
  typedef struct {
    std::string                 swathname;
    std::string                 gfldname;
    std::vector<int32>          dims;
    std::vector<std::string>    dimnames;
    int32                       type;
    void*                       geomap;
  } GeoField ;

/**
 * Structure to hold geographic metadata about the image
 */
  typedef struct {
    std::string                 swathname;
    std::string                 fieldname;
    std::vector<int32>          dims;
    std::vector<std::string>    dimnames;
    int32                       type;
    GeoField*                   geofldlat;
    GeoField*                   geofldlon;
    std::vector<int32>          latoff;
    std::vector<int32>          lonoff;
    std::vector<int32>          latinc;
    std::vector<int32>          loninc;
  } DataField ;

  std::vector<DataField>  mDFList;
  std::vector<GeoField> mGFList;
  Projection              mProj;

  /**
   *  get info from Aster HDF file
   */
  void      parseGeoFlds( int32 swid, std::string swname );

  /**
   *  get info from Aster HDF file
   */
  std::string     getHdfGblAttr( const std::string &fname,
                                 const std::string &attrname ) const;

  /**
   * Get image aquisition date/time from Aster HDF file
   *
   * @history 2006-01-10 dls -
   *    Modified to not format the date-time if  invalid characters
   *    are found in Aster TIMEOFDAY or CALENDARDATE metadata fields.
   *    Formatting of date-time ISO 8601 compliant.
   */
  std::string   getAquiDateTime( const std::string &fname ) const;

  /**
   *  get projection info from Aster HDF file
   */
  std::string   getProjection( const std::string &fname ) const;

  /**
   *  get origin ID info from Aster HDF file
   */
  std::string   getOrigID( const std::string &fname ) const;

  /**
   *  get instrument zenith info from Aster HDF file
   */
  std::string   getInstZenith( const std::string &fname ) const;

  /**
   *  get percentage of cloud cover info from Aster HDF file
   */
  std::string   getCloudPct( const std::string &fname ) const;

  /**
   *  get sun azimuth info from Aster HDF file
   */
  std::string   getSunAzim( const std::string &fname ) const;

  /**
   *  get sun elevation info from Aster HDF file
   */
  std::string   getSunElev( const std::string &fname ) const;

  /**
   *  get image azimuth info from Aster HDF file
   */
  std::string   getImageAzim( const std::string &fname ) const;


  /**
   *  get image ID info from Aster HDF file
   * @history 2006-01-09 dls - Added function
   */
  std::string   getImageID( const std::string &fname ) const;

  /**
   *  get instrument ID info from Aster HDF file
   * @history 2006-01-09 dls - Added function
   */
  std::string   getInstID( const std::string &fname ) const;

  /**
   *  get instrument azimuth info from Aster HDF file
   * @history 2006-01-09 dls - Added function
   */
  std::string   getInstAz( const std::string &fname ) const;


  /**
   *  get band metadata info from Aster HDF file
   */
  void          getBandMetaData( ) const;

  std::vector<std::string>
  splitList( char *clist, char delim ) {
    int l, r;
    std::vector<std::string> ret;
    std::string list = clist;
    l = 0;
    while ( l < (int)list.length()-1 ) {
      r = list.find( delim, l );
      if ( r == -1 ) r = list.length();
      ret.push_back( list.substr( l, r-l ) );
      l = r+1;
    }
    return ret;
  }

  template<class DType>
  void findrange( int32 swid,
                  std::string field,
                  DType &lwr,
                  DType &upr,
                  std::vector<int32> dims );

  template<class DType>
  void maxmin( DType* data, int sz, DType &lwr, DType &upr ) {
    for ( int i=0; i < sz; i++ ) {
      if ( lwr > data[i] )
        lwr = data[i];
      if ( upr < data[i] )
        upr = data[i];
    }
  }

  template<class DType>
  void bilinterp( DType  *map,
                  std::vector<int32>  dims,
                  std::vector<int32>  offset,
                  std::vector<int32>  inc,
                  int    x,
                  int    y,
                  double &res );

  void getGeoFields( int iband );
  void getIntersect( int &track,
                     int &p1,
                     int p2,
                     int iband,
                     double searchVal,
                     bool isLon,
                     bool horizontal=true );

  void trackLineTo( int &x,
                    int &y,
                    int iband,
                    double trackVal,
                    double searchVal );

  /*
  void getULCorners( );
  void xyToLL( int iband, double &x, double &y );
  void xyToMeter( int iband, double &x, double &y );
  void meterToLL( int iband, double &x, double &y );
  void llToMeter( int iband, double &lon, double &lat );
  void llToXY( int iband, double &lon, double &lat );
  projPJ  createProj( int iband );
  */

  bool between( double a, double b, double c ) {
    if ( a < b )
      return( c > a && c < b );
    else
      return( c < a && c > b );
  }

public:
  ~HdfEosFrmt( );

  /**
   * Open an HDF image and pull out associated metadata.
   *
   * @history 2006-01-09 dls -
   *          Altered open function to pull image metadata and fill in
   *          ImageInf structure
   *
   * @history 2005-08-20 dls
   *          modified to not overwrite image acquisition date
   *          read in from project file
   */
  virtual bool open( std:: string fname );

  virtual void getLL( int iband, double &i, double &j, bool toWGS84 );
  virtual void getLL( int iband, double &i, double &j ) {
    if ( iband >= 0 && iband < (int)mBandSet.size() &&
         mBandSet[iband].proj != "" ) {
      xyToLL( iband, i, j );
      return;
    }
    getLL( iband, i, j, true );
  }

  virtual void getXY( int iband, double &i, double &j );
  virtual void* getBand( int iband,
                         int x1,
                         int y1,
                         int w1,
                         int h1,
                         int w2,
                         int h2,
                         ScaleType = SCALE_BI ) const;
};

#endif





