#ifndef HISTOGRAM_H
#define HISTOGRAM_H

#include <vector>

using namespace std;

/**
 * A value and count associated with the number of occurences of val.
 * This is used with Histogram to provide a DN count on an image.
 *
 * @author Dan Mingus 2002-10-09 modified
 * @history 2004-10-21 Deborah Lee Soltesz - Formatted documentation for doxygen
 *
 * @ingroup histogram
 */
class Hash {
public:
  double  mVal;       //!< value
  int     mCnt;       //!< count

public:

/**
 *           Sets the value to val and the count to 1.
 */
  Hash( double val ) {
    mVal = val;
    mCnt = 1;
  }


/**
 *           Increment the count for this value.
 */
  void    incCnt( ) {
    mCnt++;
  }


/**
 *           Gets the value.
 */
  double  getVal( ) {
    return mVal;
  }


/**
 *           Gets the count.
 */
  int     getCnt( ) {
    return mCnt;
  }
};

/**
 * The Histogram class is designed
 * to provide an efficient data type independent routine for calculating
 * the histogram of an array and allow easy access to it with the
 * use of class Hash, which is a basic val cnt mapping.
 *
 * @history 2002-10-09 Dan Mingus - created/modified
 *
 * @ingroup histogram
 *
 */
class Histogram {
private:
  int       mLen;           // LENGTH OF THE DATASET
  int       mMaxCnt;        // MAX COUNT OF ALL VALUES
  double    mMin;           // MIN DATA VALUE
  double    mMax;           // MAX DATA VALUE

  vector<Hash>    mHist;      // THE HISTOGRAM

/**
 *           Add a new value to the histogram.  find( double val )
 *           would be called first to make sure the value is not
 *           allready present.
 */
  void insert( double val ) {
    Hash hash( val );
    mHist.insert( (vector<Hash>::iterator)findGt( val ), hash );
  }

/**
 *           Find a Hash with "val" value.  Non-linear search.
 */
  Hash* find( double val ) {
    int l;  // LEFT POS IN ARRAY
    int r;  // RIGHT POS IN ARRAY
    int m;  // MID POS BETWEEN LEFT & RIGHT

    // ----------------------------------------------------------------
    // INITIALIZE POSITIONS
    l = 0;
    r = mHist.size() -1;

    // NOTHING TO SEARCH?
    if ( r == -1 )
      return NULL;

    m = r / 2;
    // ----------------------------------------------------------------

    // ----------------------------------------------------------------
    // FIND THE POSITION OF THE Hash WITH val
    while ( l < r-1 ) {
      if ( val > mHist[m].mVal )
        l = m;
      else if ( val < mHist[m].mVal )
        r = m;
      else
        return &mHist[m];

      m = ((r - l) / 2) + l;
    }

    if ( val == mHist[l].mVal )
      return &mHist[l];

    if ( val == mHist[r].mVal )
      return &mHist[r];
    // ----------------------------------------------------------------

    // THERE'S NO Hash WITH val
    return NULL;
  }

/**
 *           Find the first Hash with mVal greater than val.
 *           Non-linear search.
 */
  Hash* findGt( double val ) {
    int l;  // LEFT POS IN ARRAY
    int r;  // RIGHT POS IN ARRAY
    int m;  // MID POS BETWEEN LEFT & RIGHT

    // ----------------------------------------------------------------
    // INITIALIZE POSITIONS
    l = 0;
    r = mHist.size() -1;

    // NOTHING TO SEARCH?
    if ( r == -1 )
      return NULL;

    m = r / 2;
    // ----------------------------------------------------------------

    // ----------------------------------------------------------------
    // FIND THE POSITION OF THE Hash WITH val
    while ( l < r-1 ) {
      if ( val > mHist[m].getVal() )
        l = m;
      else
        r = m;

      m = ((r - l) / 2) + l;
    }

    if ( val <= mHist[l].getVal() )
      return &mHist[l];

    if ( val <= mHist[r].getVal() )
      return &mHist[r];
    // ----------------------------------------------------------------

    // ***************************************************
    // This seems wrong.  If r==0 and it gets here it should
    // crash. FIX!
    // ***************************************************
    return &mHist[r+1];
  }

public:

  Histogram( ) {
    mLen = 0;
    mMaxCnt = 0;
    mMin = 0;
    mMax = 255;
  }
/**
 *           This function calculates the histogram for the given
 *           dataset.  The dataset should be a numeric array.  This
 *           will loop through every value in the dataset and store
 *           it's count in a Hash.  When the routine ends, the
 *           member hash will be an array of value count mappings,
 *           or the histogram of this dataset.
 */
  template<class DType>
  void calcHist( DType *data, int len  ) {
    Hash    *hash = NULL;   // THE RESULTING HISTOGRAM
    int     iVal = 0;       // ARRAY INDEX

    mLen = len;

    // ----------------------------------------------------------------
    // LOOP THROUGH DATASET & CALC HISTOGRAM
    for ( ; iVal < mLen; iVal++ ) {
      hash = find( (double)data[iVal] );

      if ( hash )
        hash->incCnt();
      else
        insert( (double)data[iVal] );
    }
    // ----------------------------------------------------------------

    // ----------------------------------------------------------------
    // DETERMINE THE HASH WITH THE GREATEST COUNT
    mMaxCnt = -1;
    for ( unsigned int i=0; i < mHist.size(); i++ )
      if ( mHist[i].mCnt > mMaxCnt )
        mMaxCnt = mHist[i].mCnt;
      // ----------------------------------------------------------------

      // SINCE THIS IS AN ORDERED SET, THE MIN & MAX ARE ON THE ENDS
    mMin = mHist[0].mVal;
    mMax = mHist[mHist.size()-1].mVal;

  }

/**
 *           Get the histogram that was calculated with calcHist.
 */
  vector<Hash> getHist( ) {
    return mHist;
  }


/**
 *           Get the minimum value of the calculated dataset
 */
  double getMin( ) {
    return mMin;
  }


/**
 *           Get the maximum value of the calculated dataset
 */
  double getMax( ) {
    return mMax;
  }


/**
 *           Get the length of the calculated dataset.
 *           \deprecated This function may be useless.
 *
 */
  int    getLen( ) {
    return mLen;
  }


/**
 *           Get the maximum count in the histogram
 */
  int    getMaxCnt( ) {
    return mMaxCnt;
  }
};
#endif



