#ifndef HISTOGRAMDLG_H
#define HISTOGRAMDLG_H

// QT
#include <qwidget.h>
#include <qcolor.h>
#include <qmenubar.h>
#include <qaction.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpointarray.h>
#include <qcursor.h>
#include <qapplication.h>

// STDLIB
#include <vector>
#include <stack>
#include <cstdio>

// LOCAL
#include "histogramdatasource.h"
#include "histogramuser.h"


/**
 *   This is a dialog for displaying the histogram the viewed
 *   dataset and allowing contrast stretching.  This provides
 *   the UI for adjusting a piece-wise linear stretch in reference
 *   to the dataset's histogram.
 *
 *    LAST MODIFIED:
 *      Oct 9, 2002
 *      Dan Mingus
 *
 */
class HistogramDataSource;

/**
 *  Structure to hold important display dimensions & positions within the
 *  graph.
 */
typedef struct {
  int scaleW;         //!< the graph's scale width at left & bottom
  int graphBorderW;   //!< the graph's border at top & right
  int topY;           //!< the top of the display area(menubar height)
  int winW;           //!< the window's width
  int winH;           //!< the window's height
  int graphW;         //!< the graph's width
  int graphH;         //!< the graph's height
  int graphZeroX;     //!< the graph's x=zero pos
  int graphMaxX;      //!< the graph's x=(max x value) pos
  int graphZeroY;     //!< the graph's y=zero pos
  int graphMaxY;      //!< the graph's y=(max y value) pos
} DspDims;

/**
 *  Structure to hold the mouse state.
 */
typedef struct {
  int  mPointMoving;  //!< state of stretch points' motion
  bool mLeftDown;     //!< left mouse button down
  bool mMidDown;      //!< middle mouse button down
  bool mRightDown;    //!< the right mouse button down
  int  mMouseX;       //!< the mouse's x pos
  int  mMouseY;       //!< the mouse's y pos
} MouseState;


/**
 *    Structure to hold the histogram datasources, histogram sets and relative
 *    information.
 */
typedef struct {
  vector<HistogramDataSource*> HDS;         //!< the datasource set
  vector<QAction*>             actSet;      //!< the dataset selection actions in menu
  int                          curHDS;      //!< index of current hds
  HistogramSet                 histSet;     //!< the histogram set
  int                          curHist;     //!< index of the current hist in histset
  QPointArray                  dataPnts[3]; //!< the polygons representing this histset
} HDSSet;

/**
 *    Structure to hold the stretch line history stacks, redo stack, & state
 */
typedef struct {
  stack<QPointArray>  lineSet[3]; //!< history stack of all lines(top line is current)
  stack<QPointArray>  redoSet;    //!< the redo stack
  int                 curLine;    //!< index to the current line(1-3 if rgb)
  QPointArray         *curArr;    //!< convinience ptr to the current point array
  bool                isChanging; //!< state of line action
  bool                isMerged;   //!< merged display state
} StretchLineSet;

/**
 * A user interface dialog that presents the user with
 * histogram information in graph form and allows the user
 * to generate nodes on it to produce a piecewise linear
 * stretch.  This results in contrast stretching in the
 * views.  The user can select between the different color
 * channels in the image or all at the same time.
 */
class HistogramDlg : public QWidget {
  Q_OBJECT
  public:

  protected:
  QMenuBar            *mMenuBar;   //!< the menubar

  MouseState          mMouse;      //!< mouse state
  StretchLineSet      mSLSet;      //!< stretch lines
  HDSSet              mHDS;        //!< datasource set

  HistogramUser       *mHistUser;  //!< the user of this histogram

  vector<QAction*>    mPercentAct; //!< the stretch % selection in menu
  QAction             *mMergeAct;  //!< the merge toggle action in menu
  QActionGroup    *mDSActGroup;
  QPopupMenu      *mDSMenu;
  vector<QAction*>    mColorAct;   //!< the color selection action in menu

  DspDims             mDspDims;    //!< display dimensions
  DspDims             mOldDspDims; //!< previous display dimension if resized

  DspDims calcDspDims( );

  void init( );
  void buildMenuBar( );
  bool mouseInGraph( );
  int  mouseOnPoint( );
  void createDataPnts( );
  void alertUser( );
  void initPntsFromUser( );

public:
  HistogramDlg( QWidget *parent );
  HistogramDlg( QWidget *parent,
                vector<HistogramDataSource*> hds,
                HistogramUser *huser=NULL );

  void mousePressEvent( QMouseEvent * );
  void mouseReleaseEvent( QMouseEvent * );
  void mouseMoveEvent( QMouseEvent * );
  void mouseDoubleClickEvent( QMouseEvent * );
  void paintEvent( QPaintEvent *pe );
  void resizeEvent( QResizeEvent *re );

  void addHDS( HistogramDataSource *hds );
  void setHDS( vector<HistogramDataSource*> hds );
  void setUser( HistogramUser *user );

public slots:
  void        changeToRed( );
  void        changeToBlue( );
  void        changeToGreen( );
  void        changeToAll( );
  void        changeDataSource( );
  void        bestFit( );
  void        mergeAll( );
  void        clearPoints( );
  void        undoToLast( );
  void        redoToPrev( );
  void        reloadHistSet( );
  void        calcPercent( );
};

#endif



