#ifndef __IMAGE_H
#define __IMAGE_H

#include "windows_defs.h"

#include <qimage.h>
#include <qpixmap.h>

#include <string>
#include <vector>

#include "imageformat.h"
#include "inmemformat.h"
#include "imagestate.h"
#include "shape.h"

/**
 * This provides access to the ImageFormats through a single
 * interface.  A static open routine test all available formats
 * for usability with the given filename.
 *
 * Notes: Basically an abstract factory + manager or possibly proxy
 * of a sort to instantiate the correct subclass of ImageFormat
 * (abstract parent to instrument-specific and otherwise varying image
 * data file format handling subclasses) and possibly provide a
 * generic interface to the data. In some cases, it appears
 * Image object is bypassed and the programmer goes directly to
 * the ImageFormat object to gain access to the data (see Threshhold
 * plug-in). [2006-01-16 Deborah Lee Soltesz]
 *
 * @author Dan Mingus
 */

class GV_EXPORT Image {
public:
  typedef enum {
    NEAREST_NEIGHBOR,
    BILINEAR,
    BICUBIC
  } ResampleMethod;

  typedef enum {
    LSAT7_GEOTIFF,
    LSAT7_NLAPS,
    LSAT7_FASTL7A,
    GENERIC
  } MultiFileType;

  static std::string MFF_NAMES[];
  static int NUM_MFF;

protected:
  ImageFormat*                mImgFrmt;
  int                         mWidth;
  int                         mHeight;
  ImageState                  mState;
  InMemFormat                 *mMemFrmt;

  template<class DType>
  unsigned char* scaleToUC( DType *data,
                            double min,
                            double max,
                            int len );

  void makeRGBA( unsigned char* imbits,
                 unsigned char* rData,
                 unsigned char* gData,
                 unsigned char* bData,
                 int newWidth,
                 int newHeight );


public:
  Image( ImageFormat * );
  ~Image( );

  static Image* openImage( std::string fname,
                           MultiFileType type=GENERIC );

  QImage* getRect(
                 int x1,
                 int y1,
                 int w1,
                 int h1,
                 int w2,
                 int h2,
                 ImageFormat::ScaleType st = ImageFormat::SCALE_BI );

  int width( );
  int height( );

  int width( int iband );
  int height( int iband );

  std::vector<ImageFormat::BandInf>& getBandInfSet( ) const;
  std::string getName( ) const;

  /**
   * The values of x and y, assumed to be in XY coordinates,
   * are modified to the corresponding latitude and longitude
   *
   * @param[in,out] x The X coordinate of a point in the image.
   *                  When the function returns, x holds the longitude
   *                  corresponding to the input XY location.
   *
   * @param[in,out] y The Y coordinate of a point in the image.
   *                  When the function returns, y holds the latitude
   *                  corresponding to the input XY location.
   */
  void getLL( double &x, double &y );

  /**
   * The values of x and y, assumed to be latitude-longitude coordinates,
   * are modified to the corresponding XY coordinates.
   *
   * @param[in,out] x The longitude coordinate of a point in the image.
   *                  When the function returns, x holds the X coordinate
   *                  corresponding to the input lat-lon location.
   *
   * @param[in,out] y The latitude coordinate of a point in the image.
   *                  When the function returns, y holds the Y coordinate
   *                  corresponding to the input lat-lon location.
   *
   */
  void getXY( double &x, double &y );

  const ImageInf& getImageInf( ) {
    return mImgFrmt->getImageInf();
  }

  void setCornerPnts( int iband, ImageInf &inf ) {
    mImgFrmt->setCornerPnts( iband, inf );
  }

  void setCornerPnts( ImageInf &inf ) {
    mImgFrmt->setCornerPnts( mState.mRed, inf );
  }

  ImageState getImageState( ) {
    return mState;
  }
  void setImageState( ImageState &state ) {
    mState = state;
    if ( mMemFrmt )
      mMemFrmt->stateToMem();
  }

  void setBandSel( ImageState &state ) {
    mState.mRed   = state.mRed;
    mState.mGreen = state.mGreen;
    mState.mBlue  = state.mBlue;

    if ( mMemFrmt )
      mMemFrmt->stateToMem();
  }

  void setMemBuf( bool state );

  ImageFormat* getFormat( ) {
    return mImgFrmt;
  }
};

#endif




