#ifndef __IMAGEFORMAT_H
#define __IMAGEFORMAT_H

#include "windows_defs.h"

#include <string>
#include <vector>
#include <proj_api.h>

#include "imageinf.h"

#ifndef M_PI
#define M_PI 3.1415926
#endif

/**
 * An abstract class that provides an interface to
 * a basic image format.  Requirements are things like
 * I/O and georeferencing as well as general information
 * about the image.
 *
 * To add a new image format to glimsview, all one needs
 * to do is extend this class, override and implement
 * its abstract routines, and fill in the mImageInf member.
 * A recompilation of the source will be necessary.
 *
 * Notes: An additional feature to glimsview would be to reimplement
 * this as another plugin interface that dynamically loads
 * the image formats available to the application on start that
 * would be stored in a particular directory, i.e. ./formats/.
 * This would allow anyone to append their own file format without
 * the source. [Dan Mingus]
 *
 * @author Dan Mingus
 */

class GV_EXPORT ImageFormat                                                         {
public:
  typedef enum {
    SCALE_NN = 1,
    SCALE_BI,
    SCALE_BC
  } ScaleType;

  typedef enum {
    NOT_DEFINED,
    INT8,
    UINT8,
    INT16,
    UINT16,
    INT32,
    UINT32,
    INT64,
    UINT64,
    FLOAT16,
    FLOAT32,
    FLOAT64
  } DataType;

  /**
   * Structure to hold metadata about the bands in the image
   */
  typedef struct {
    int             width;
    int             height;
    DataType        type;
    std::string     name;
    double          res;
    double          swlen;
    double          ewlen;
    double          max;
    double          min;
    int             ndims;
    std::string     proj;
    int             UTMZ;
    std::vector<std::string> parms;
    double          rot;
    double          ulx;
    double          uly;
  } BandInf;

private:
  char *mParms[15];

protected:
  std::string                 mFName;
  std::vector<BandInf>        mBandSet;
  ImageInf                    mImageInf;

  void xyToLL( int iband, double &x, double &y );
  void xyToMeter( int iband, double &x, double &y );
  void meterToLL( int iband, double &x, double &y );
  void llToMeter( int iband, double &lon, double &lat );
  void llToXY( int iband, double &lon, double &lat );
  projPJ  createProj( int iband );
  void getULCorners( );

public:
  ImageFormat( );
  virtual ~ImageFormat( );

  virtual bool open( std::string fname ) =0;
  virtual void* getBand( int iband,
                         int x1,
                         int y1,
                         int w1,
                         int h1,
                         int w2,
                         int h2,
                         ScaleType st = SCALE_BI ) const =0;

  virtual void getLL( int iband,
                      double &i,
                      double &j ) =0;

  virtual void getXY( int iband,
                      double &i,
                      double &j ) =0;


  // void getImageInf( GlimsDataset::ImageInf &imginf );
  std::vector<BandInf>& getBandInfSet( );
  unsigned int getNumBands( ) {
    return mBandSet.size();
  }
  const BandInf& getBandInf( int iband );
  std::string getName( ) const;

  static std::string dtypeToString( DataType dt );
  static int dtypeSize( DataType dt );
  static bool isFloat( DataType dt );

  // 8 BIT IMAGE SCALING ROUTINES
  char* scaleData( char **orig,
                   int ow,
                   int oh,
                   int nw,
                   int nh,
                   DataType dt,
                   ScaleType st,
                   bool del = true ) const;

  char* scaleData( char **orig,
                   int ox,
                   int oy,
                   int ow,
                   int oh,
                   int fw,
                   int fh,
                   int nw,
                   int nh,
                   DataType dt,
                   ScaleType st,
                   bool del = true ) const;

  template<class DType> DType*
  scaleDT( DType* orig,
           int ox,
           int oy,
           int ow,
           int oh,
           int fw,
           int fh,
           int nw,
           int nh,
           ScaleType st ) const;

  template<typename DType> DType*
  scl_nearestneighbor( DType* orig,
                       int ox,
                       int oy,
                       int ow,
                       int oh,
                       int fw,
                       int fh,
                       int nw,
                       int nh ) const;

  template<class DType> DType*
  scl_bilinear( DType* orig,
                int ox,
                int oy,
                int ow,
                int oh,
                int fw,
                int fh,
                int nw,
                int nh ) const;

  unsigned char* scl_bicubic( unsigned char* orig,
                              int ow,
                              int oh,
                              int nw,
                              int nh,
                              int szd ) const;

  ImageInf& getImageInf( ) {
    return mImageInf;
  }

  void setCornerPnts( int iband, ImageInf &inf );
};

#endif




