#ifndef IMAGEINFDLG_H
#define IMAGEINFDLG_H

#include <qdialog.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qpushbutton.h>

#include <string>

#include "glimsdataset.h"

using namespace std;

/**
 * A user interface dialog providing editability
 * to a ImageInf.  This needs improvement to allow
 * more access to the fields within ImageInf.  Currently,
 * only aqusition date/time is editable.  But this is
 * also the only field in the images.shp file that is
 * not able to be null in the GLIMS database.
 *
 * @history 2005-08-20 deborah lee soltesz -
 *          added & to mImgInf declaration so it would
 *          hold its data correctly
 *
 * @history 2006-01-09 deborah lee soltesz -
 *          Added handling of all the image metadata:
 *          more input fields, storing data in ImageInf
 *          structure, etc. Added "check" button to
 *          validate date/time stamp for Image Acquisition.
 *
 * @whatsnew 2006-01-21 -
 *          More image metadata is displayed and can be
 *          edited to fill in missing or correct erroneous
 *          information. "Check" button for the image
 *          acquisition date allows user to validate
 *          format of the date.
 *
 * @todo 2006-01-09 dls -
 *          Create better DateTime edit field to force users
 *          to provide a date/time that can be easily
 *          written in the correct ISO format.
 *
 * @todo 2006-01-23 dls -
 *    Need to confirm all the information required by
 *    the GLIMS database is being captured and stored
 */

class ImageInfDlg : public QDialog {
  Q_OBJECT

protected:
  ImageInf        &mImgInf;

  QLineEdit *fne ;         //!< image filename (or image location)
  QLineEdit *le ;          //!< image acquisition date/time
  QLineEdit *imgID ;       //!< image identifier
  QLineEdit *instID ;      //!< instrument identifier
  QLineEdit *origID ;      //!< origin identifier
  QLineEdit *cLat ;        //!< center latitude
  QLineEdit *cLon ;        //!< center longitude
  QLineEdit *cLatUnc ;     //!< center latitude UNC (?)
  QLineEdit *cLonUnc ;     //!< center longitude UNC (?)
  QLineEdit *imgAz ;       //!< image azimuth
  QLineEdit *cloudPct ;    //!< percentage cloud cover
  QLineEdit *sunAz ;       //!< sun azimuth
  QLineEdit *sunElev ;     //!< sun elevation
  QLineEdit *instAz ;      //!< instrument azimuth
  QLineEdit *instZen ;     //!< instrument zenith
  QLineEdit *proj ;        //!< projection

  QPushButton   *checkLe ;     //!< validates date/time stamp

  /**
   * Do the characters in the input string represent
   * a complete and valid ISO 8601 date/time string?
   * This function does not do a very stringent test,
   * just simply checks that there are the correct
   * range of characters in each slot and there's
   * a sufficient number of digits to create a valid
   * date/time string.
   *
   * See http://www.w3.org/TR/NOTE-datetime
   *
   * @todo 2006-01-21 dls -
   *       Make this function more stringent and robust.
   *       Invalid dates (e.g. 2006-19-32) should return
   *       false.
   *
   */
  bool validIso8601DateTime (QString &str) ;

public:
  ImageInfDlg( ImageInf & );
  void loadData () ;
  string getAquiDate () { return mImgInf.mAquiDateTime ; }

public slots:
  /**
   * Image aquisition date/time changed
   */
  void aquchanged( const QString & );

  /**
   * Image aquisition date/time completed
   */
  void aquCompleted( );

  /**
   * Image filename changed
   */
  void fnchanged( const QString &str );

  /**
   * Image ID changed
   * @history 2006-01-08 dls - Added new function
   */
  void imgIdChanged(const QString &str );

  /**
   * Instrument ID changed
   * @history 2006-01-08 dls - Added new function
   */
  void instIdChanged(const QString &str );

  /**
   * Origin ID changed
   * @history 2006-01-08 dls - Added new function
   */
  void origIdChanged(const QString &str );

  /**
   * Center latitude changed
   * @history 2006-01-08 dls - Added new function
   */
  void cLatChanged(const QString &str );

  /**
   * Center longitude changed
   * @history 2006-01-08 dls - Added new function
   */
  void cLonChanged(const QString &str );

  /**
   * Center latitude uncertainty changed
   * @history 2006-01-08 dls - Added new function
   */
  void cLatUncChanged(const QString &str );

  /**
   * Center longitude uncertainty changed
   * @history 2006-01-08 dls - Added new function
   */
  void cLonUncChanged(const QString &str );

  /**
   * Image azimuth changed
   * @history 2006-01-08 dls - Added new function
   */
  void imgAzChanged(const QString &str );

  /**
   * Percentage cloud cover changed
   * @history 2006-01-08 dls - Added new function
   */
  void cloudPctChanged( const QString &str );

  /**
   * Sun azimuth changed
   * @history 2006-01-08 dls - Added new function
   */
  void sunAzChanged(const QString &str );

  /**
   * Sun elevation changed
   * @history 2006-01-08 dls - Added new function
   */
  void sunElevChanged(const QString &str );

  /**
   * Instrument azimuth changed
   * @history 2006-01-08 dls - Added new function
   */
  void instAzChanged(const QString &str );

  /**
   * Instrument zenith changed
   * @history 2006-01-08 dls - Added new function
   */
  void instZenChanged(const QString &str );

  /**
   * Image projection information changed
   * @history 2006-01-08 dls - Added new function
   */
  void projChanged(const QString &str );
};

#endif




