#include "pluginset.h"
#include <qmessagebox.h>

PluginSet::PluginSet( QWidget *parent,
                      QPopupMenu *menuparent ) :
QWidget( parent ),
PLUGCFG_FNAME( "plugins/plugcfg.xml" ),
mMenuParent( menuparent ) {
  mMenuGroup  = NULL;
  mProj       = NULL;

  QAction *action;
  action = new QAction( tr("Configure"), tr("&Configure"), 0, this );
  action->addTo( menuparent );
  connect( action,
           SIGNAL( activated() ),
           this,
           SLOT( showPluginCfg() ) );

  menuparent->insertSeparator();

  buildWidget();
  resize( 300, 250 );

  fromXMLFile( PLUGCFG_FNAME, "PluginConfig" );
  loadList();
  loadMenu();
}

void PluginSet::buildWidget( ) {
  setCaption( "Plugin Configurator" );

  QVBoxLayout *main = new QVBoxLayout( this );
  main->setAutoAdd( true );

  QGroupBox *gbox;
  QPushButton *pbut;

  gbox = new QGroupBox( 1, Qt::Vertical, "Plugins", this );
  mPlugList = new QListView( gbox );
  mPlugList->addColumn( "Name" );
  mPlugList->addColumn( "File", 150 );
  mPlugList->setMinimumSize( 200, 75 );
  mPlugList->setMaximumHeight( 125 );
  connect( mPlugList,
           SIGNAL( selectionChanged( QListViewItem * ) ),
           this,
           SLOT( selectedPluginChanged( QListViewItem * ) ) );

  gbox = new QGroupBox( 1, Qt::Vertical, "Description", this );
  mDescBox = new QMultiLineEdit( gbox );
  mDescBox->setMinimumSize( 200, 75 );
  mDescBox->setWordWrap( QMultiLineEdit::WidgetWidth );

  QHBox *hbox = new QHBox( this );
  pbut = new QPushButton( "Import", hbox );
  connect( pbut,
           SIGNAL( clicked() ),
           this,
           SLOT( importPlugin() ) );

  pbut = new QPushButton( "Rescan", hbox );
  connect( pbut,
           SIGNAL( clicked() ),
           this,
           SLOT( rescanDir() ) );

  pbut = new QPushButton( "Close", hbox );
  connect( pbut,
           SIGNAL( clicked() ),
           this,
           SLOT( hide() ) );
}

bool PluginSet::toXML( QDomDocument &doc,
                       QDomElement  &elem,
                       std::string id ) {
  QDomElement base = doc.createElement( "PluginSet" );
  if ( id != "" )
    base.setAttribute( "id", (char*)id.c_str() );
  elem.appendChild( base );

  for ( int i=0; i < (int)mPlugSet.size(); i++ )
    mPlugSet[i].toXML( doc, base );

  return true;
}

bool PluginSet::fromXML( QDomElement &elem ) {
  if ( elem.isNull() )
    return false;

  if ( std::string( "PluginSet" ).compare( (const char*)elem.tagName() ) )
    return false;

  QDomNodeList list = elem.elementsByTagName( "Plugin" );
  QDomElement e;
  for ( int i=0; i < (int)list.count(); i++ ) {
    Plugin plug;
    e = list.item( i ).toElement();
    plug.fromXML( e );
    mPlugSet.push_back( plug );
  }

  return true;
}

void PluginSet::importPlugin( ) {
  QString fname =
  QFileDialog::getOpenFileName( "", "", NULL, "", "Plugin File Chooser" );
  if ( fname.isNull() )
    return;

  Plugin plug;
  if ( plug.loadFromFile( (const char*)fname ) ) {
    for ( int i=0; i < (int)mPlugSet.size(); i++ ) {
      if ( mPlugSet[i].getName() == plug.getName() ) {
        QMessageBox::information(
                                NULL,
                                "Plugin Load Error",
                                "Plugin with same name already exists" );
        return;
      }
    }
    mPlugSet.push_back( plug );
  } else {
    QMessageBox::information( NULL,
                              "Plugin Load Error",
                              "Could not import plugin" );
    return;
  }

  loadList();
  toXMLFile( PLUGCFG_FNAME, "PluginConfig" );
  loadMenu();
}

void PluginSet::rescanDir( ) {

}

void PluginSet::loadMenu( ) {
  if ( mMenuGroup ) {
    mMenuGroup->removeFrom( mMenuParent );
    delete mMenuGroup;

  }

  mMenuGroup = new QActionGroup( mMenuParent );

  for ( int i=0; i < (int)mPlugSet.size(); i++ ) {
    ViewPopupAction *action =
    new ViewPopupAction(
                       mPlugSet[i].getName().c_str(),
                       mPlugSet[i].getName().c_str(),
                       0,
                       mMenuGroup );
    // action->addTo( mMenuParent );
    connect( action,
             SIGNAL( selected( ViewPopupAction* ) ),
             this,
             SLOT( pluginChosen( ViewPopupAction* ) ) );
  }

  mMenuGroup->addTo( mMenuParent );
}

void PluginSet::showPluginCfg( ) {
  this->show();
  this->setFocus();
}

void PluginSet::loadList( ) {
  mPlugList->clear();

  for ( int i=0; i < (int)mPlugSet.size(); i++ ) {
    new QListViewItem( mPlugList,
                       mPlugSet[i].getName().c_str(),
                       mPlugSet[i].getFile().c_str() );
  }
}

void PluginSet::pluginChosen( ViewPopupAction *action ) {
  std::string name = (const char*)action->menuText();
  if ( name == "" || mProj == NULL )
    return;

  int i=0;
  for ( ; i < (int)mPlugSet.size(); i++ ) {
    if ( name == mPlugSet[i].getName() )
      break;
  }

  if ( i >= (int)mPlugSet.size() )
    return;


  mPlugSet[i].execPlugin( mProj );

}

void PluginSet::selectedPluginChanged( QListViewItem *item ) {
  QListViewItem *selitem;
  int i=0;

  selitem = mPlugList->firstChild();
  while ( item != selitem && i < mPlugList->childCount() ) {
    i++;
    selitem = selitem->nextSibling();
  }

  if ( i >= mPlugList->childCount() && i < (int)mPlugSet.size() )
    return;

  mDescBox->setText( mPlugSet[i].getDesc().c_str() );
}




