#include "selectionset.h"

  SelectionSet::SelectionSet() {
    resetBox();
  }

  SelectionSet::~SelectionSet() {;}

  /*
   * Overload [] to make selection set behave like
   * an array of SelObjs.
   * Return the information about the selected object
   * as stored in the list at index iobj. Useful for
   * iterating through all the selected vector objects.
   *
   * @param iobj   index into the array of selected objects.
   *
   * @return SelObj is the information for a single selected
   *         object in the list of all the objects the user has
   *         selected on the screen.
   */
  SelectionSet::SelObj& SelectionSet::operator[]( int iobj ) {
    return mSelObj[iobj];
  }

  /*
   * Deselect all the vector objects.
   */
  void SelectionSet::clear() {
    mSelObj.clear();
    emit selectionChanged() ;
  }

  /*
   * How many vector objects are in the selection list?
   *
   * @return The number of vector objects in the selection list.
   */
  int SelectionSet::size() {
    return mSelObj.size();
  }

  /*
   * Add a new object to the list of vector objects
   * selected by the user.
   *
   * @param obj    a single vector object to add to the selection list.
   */
  void SelectionSet::add( SelObj &obj ) {
    mSelObj.push_back( obj );
    emit selectionChanged() ;
  }

  /*
   * Get the structure holding the information about the
   * selection boundary the user has defined.
   *
   * @return structure holding the information about the
   *         selection boundary the user has defined.
   */
  SelectionSet::SelBox& SelectionSet::selBox() {
    return mBox;
  }

  /*
   * Set the status of the user defining a selection boundary.
   * Change the status of whether or not the user is
   * defining (or has defined) a boundary for selecting
   * vector objects.
   *
   * @param sel    boolean
   *               true: selection boundary is being set or has been set
   *               false: no selection boundary
   */
  void SelectionSet::setSelecting( bool sel ) {
    mSelecting = sel;
  }

  /*
   * Is a selection boundary being set or has it been
   * set?
   *
   * @return true or false
   */
  bool SelectionSet::isSelecting( ) {
    return mSelecting;
  }

  /*
   * Reset the selection boundary.
   * Wipe out the selection boundary and set its status
   * to "no selection" (mSelecting = false).
   */
  void SelectionSet::resetBox() {
    mBox.x = mBox.y = mBox.w = mBox.h = 0;
    mSelecting = false;
  }

  /*
   * Remove vector object "i" from list of selected objects.
   *
   * @param i      index of object to remove from list
   */
  void SelectionSet::erase( int i ) {
    mSelObj.erase( mSelObj.begin() + i );
    emit selectionChanged() ;
  }

  /*
   * Is this object selected?
   * Is this object on the list of selected objects?
   *
   * @param iobj   an object to look for in list of selected objects
   *
   * @return whether or not the object is selected
   */
  bool SelectionSet::isSelObj( int iobj ) {
    for ( unsigned int isel=0; isel < mSelObj.size(); isel++ ) {
      if ( mSelObj[isel].obj == iobj )
        return true;
    }
    return false;
  }

  /*
   * Is this node selected?
   *
   * @param iobj   object the node is a part of
   * @param inode  a Node to look for in the selected list
   *
   * @return whether or not the node is selected
   */
  bool SelectionSet::isSelNode( int iobj, int inode ) {
    unsigned int isel;
    for ( isel=0; isel < mSelObj.size(); isel++ ) {
      if ( mSelObj[isel].obj == iobj )
        break;
    }

    if ( isel >= mSelObj.size() )
      return false;

    for ( int in=0; in < (int)mSelObj[isel].nodes.size(); in++ ) {
      if ( mSelObj[isel].nodes[in] == inode )
        return true;
    }
    return false;
  }



