#ifndef __SELECTIONSET_H
#define __SELECTIONSET_H

#include <qobject.h>
#include <vector>

/**
 * A group of user selections of vectors objects (points and lines).
 * The user will select vector objects on the screen and these selections
 * are stored here as integer indices into the datasets themselves.
 *
 * @author Dan Mingus
 *
 * @history 2004-12-04 Deborah Lee Soltesz -
 *                     Modified this class to inherit from QObject, enabling
 *                     it to have signals and slots. This allows SelectionSet
 *                     to announce (signal) when it has been changed so that
 *                     other classes can refresh, repaint, reload, and so forth.
 */
class SelectionSet : public QObject {
  Q_OBJECT
public:
/**
 * A selected vector element.
 */
  typedef struct {
    std::vector<int>    nodes;
    int                 obj;
  } SelObj;

/**
 * User on-screen selection box or boundary.
 * Structure to hold selection box (marquis, selection boundary)
 * drawn by the user onto one of the views.
 */
  typedef struct {
    double  x;
    double  y;
    double  w;
    double  h;
  } SelBox;

protected:
  std::vector<SelObj>     mSelObj;    //!< array of selected vector elements
  SelBox                  mBox;       //!< selection box drawn by user
  bool                    mSelecting; //!< is the user selecting (?)

public:

  SelectionSet() ;

  SelectionSet(const SelectionSet& copyMe)
  : QObject () {
    mBox = copyMe.mBox ;
    mSelecting = copyMe.mSelecting ;
    mSelObj = copyMe.mSelObj ;
  }

/**
 * Copy
 */
  void operator=( const SelectionSet& copyMe ) {
    mBox = copyMe.mBox ;
    mSelecting = copyMe.mSelecting ;
    mSelObj = copyMe.mSelObj ;
  }

  ~SelectionSet() ;

  /**
   * Overload [] to make selection set behave like
   * an array of SelObjs.
   * Return the information about the selected object
   * as stored in the list at index iobj. Useful for
   * iterating through all the selected vector objects.
   *
   * @param iobj   index into the array of selected objects.
   *
   * @return SelObj is the information for a single selected
   *         object in the list of all the objects the user has
   *         selected on the screen.
   */
  SelectionSet::SelObj& operator[]( int iobj ) ;

  /**
   * Deselect all the vector objects.
   */
  void clear() ;

  /**
   * How many vector objects are in the selection list?
   *
   * @return The number of vector objects in the selection list.
   */
  int size() ;

  /**
   * Add a new object to the list of vector objects
   * selected by the user.
   *
   * @param obj    a single vector object to add to the selection list.
   */
  void add( SelObj &obj ) ;

  /**
   * Get the structure holding the information about the
   * selection boundary the user has defined.
   *
   * @return structure holding the information about the
   *         selection boundary the user has defined.
   */
  SelectionSet::SelBox& selBox() ;

  /**
   * Set the status of the user defining a selection boundary.
   * Change the status of whether or not the user is
   * defining (or has defined) a boundary for selecting
   * vector objects.
   *
   * @param sel    boolean
   *               true: selection boundary is being set or has been set
   *               false: no selection boundary
   */
  void setSelecting( bool sel ) ;

  /**
   * Is a selection boundary being set or has it been
   * set?
   *
   * @return true or false
   */
  bool isSelecting( ) ;

  /**
   * Reset the selection boundary.
   * Wipe out the selection boundary and set its status
   * to "no selection" (mSelecting = false).
   */
  void resetBox() ;

  /**
   * Remove item "i" from list of selected objects.
   *
   * @param i      index of object to remove from list, where
   *               the first object in the list is at index 0
   */
  void erase( int i ) ;

  /**
   * Is this object selected?
   * Is this object on the list of selected objects?
   *
   * @param iobj   an object to look for in list of selected objects
   *
   * @return whether or not the object is selected
   */
  bool isSelObj( int iobj ) ;

  /**
   * Is this node selected?
   *
   * @param iobj   object the node is a part of
   * @param inode  a Node to look for in the selected list
   *
   * @return whether or not the node is selected
   */
  bool isSelNode( int iobj, int inode ) ;

signals:

  // announce that the selection set has somehow changed
  void selectionChanged () ;

};

#endif




