#ifndef __LINE_H
#define __LINE_H

#include "windows_defs.h"

#include <vector>
#include <cstdlib>
#include <typeinfo>

#include <qcolor.h>
#include "xmlserializeable.h"

/**
 * This holds all necessary attributes of a vector line/point
 * display.  The default, which is produced upon creation, is
 * considered the application default.
 */

class GV_EXPORT DspAttr {
public:
  QColor          color;
  short           width;
  Qt::PenStyle    style;

  DspAttr( ) :
  color( QColor( 61, 221,37 ) ),
  width( 1 ),
  style( Qt::SolidLine ) {
  }
};

/**
 * An abstract implementation of a generic shape that
 * hold display information.  A VectorData set holds
 * shapes that are dynamically casted by code that knows
 * what kind of shape it is.
 */
class GV_EXPORT Shape {
public:
  DspAttr     mAttr;
  virtual ~Shape( ) {
  }
  virtual DspAttr& getAttr( ) {
    return mAttr;
  }
};


/**
 * A Node is a Point or a point within a line.  It holds
 * 3 dimensions of position and is used by the vector classes.
 */
class GV_EXPORT Node : public Shape, public XMLSerializeable {
public:
  double   x;
  double   y;
  double   z;

  Node( ) :
  x( 0.0 ),
  y( 0.0 ),
  z( 0.0 ) {
  }

  virtual ~Node( ) {
  }

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) {
    QDomElement base = doc.createElement( "Node" );
    if ( id != "" )
      base.setAttribute( "id", id.c_str() );
    elem.appendChild( base );

    writeDoubleTag( doc, base, "X", x );
    writeDoubleTag( doc, base, "Y", y );
    writeDoubleTag( doc, base, "Z", z );

    return true;
  }

  virtual bool fromXML( QDomElement &elem ) {
    std::string name = "Node";
    if ( name.compare( (const char*)elem.tagName() ) )
      return false;

    x = readDoubleTag( elem, "X" );
    y = readDoubleTag( elem, "Y" );
    z = readDoubleTag( elem, "Z" );
    return true;
  }
};


/**
 * This implements the common line and provides access and
 * modification routines.
 */
class GV_EXPORT Line : public Shape, public XMLSerializeable {
protected:

  std::vector<Node>   mNodeSet;

  virtual bool nodesetFromXML( QDomElement &elem );
  virtual bool nodesetToXML( QDomDocument &doc,
                             QDomElement &elem );

public:
  virtual ~Line( ) {
    mNodeSet.clear();
  }

  virtual bool fromXML( QDomElement &elem );
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" );

  // virtual DspAttr& getAttr( ) { return mAttr; }
  void setAttr( DspAttr &attr ) {
    mAttr = attr;
  }

  Node& operator[]( int inode ) const {
    return(Node&)mNodeSet[inode];
  }

  Node& getNode( int inode ) const {
    return(Node&)mNodeSet[inode];
  }

  std::vector<Node>& nodeset( ) {
    return mNodeSet;
  }
  unsigned int size( ) const {
    return mNodeSet.size();
  }
  void delNode( unsigned int inode ) {
    if ( inode < mNodeSet.size() )
      mNodeSet.erase( mNodeSet.begin() + inode );
  }
};

/**
 * This implements the common rectangle.
 */
class GV_EXPORT Rect : public Shape, public XMLSerializeable {
public:
  double    x;
  double    y;
  double    w;
  double    h;

  bool toXML( QDomDocument &doc,
              QDomElement &elem,
              std::string id="" ) {
    QDomElement base = doc.createElement( "Rect" );
    if ( id != "" )
      base.setAttribute( "id", id.c_str() );
    elem.appendChild( base );

    writeDoubleTag( doc, base, "X", x );
    writeDoubleTag( doc, base, "Y", y );
    writeDoubleTag( doc, base, "W", w );
    writeDoubleTag( doc, base, "H", h );
    return true;
  }

  bool fromXML( QDomElement &elem ) {
    if ( std::string( "Rect" ).compare( (const char*)elem.tagName() ) )
      return false;

    x = readDoubleTag( elem, "X" );
    y = readDoubleTag( elem, "Y" );
    w = readDoubleTag( elem, "W" );
    h = readDoubleTag( elem, "H" );
    return true;
  }
};

#endif




