#ifndef __VECTOREDITOR_H
#define __VECTOREDITOR_H

#include <qobject.h>
#include <qmessagebox.h>
#include <qapplication.h>

#include "vectorlayerset.h"
#include "viewset.h"

#include <algorithm>

/**
 * Allows editing of vector data of types point and line.
 * This captures viewer events produced by users such as
 * mouse click and maps them to actions on the vectors
 * themselves.
 *
 * @author Dan Mingus
 *
 * @history 2004-12-04 Deborah Lee Soltesz -
 *                     Improved the highlighting and selection to
 *                     allow the GlacierIDDlg to highlight glacier ID
 *                     selected by the VectorEditor, and to display
 *                     vector objects in the View as selected when
 *                     selected in the GlacierIDDlg.
 *
 * @history 2005-07-14 Deborah Lee Soltesz -
 *                     Modified to use modifications to the undo facility
 *                     in GLIMSLineData and keep selections and edits
 *                     synchronized.
 *
 * @history 2005-07-23 Deborah Lee Soltesz -
 *                     Added history and actions for Redo facility
 *
 * @whatsnew 2004-12-04 Lines and points associated with a glacier ID are
 *                      displayed as selected in the GlacierIDDlg when selected
 *                      on screen by the VectorEditor.
 *
 * @whatsnew 2005-07-23 Deborah Lee Soltesz
 *                      Redo!!!
 */
class VectorEditor : public QObject {
  Q_OBJECT
protected:
  VectorLayerSet  *mLayerSet ;    /*!< Set of VectorLayer objects.
                                   *
                                   *@internal Passed into VectorEditor during instantiation.
                                   */

  ViewSet         *mViewSet ;     /*!< Set of View objects.
                                   *
                                   *@internal Passed into VectorEditor during instantiation.
                                   */

  VectorData      *mData ;         /*!< VectorData from active layer, consisting of std::vectors of Shapes,
                                    *   a SelectionSet, a Stack of undo's, an Image, and some other info.
                                    *   The VectorData will be GLIMSLineData or GLIMSGIDData, both of which
                                    *   ultimately inherit from the VectorData class.
                                    *
                                    *@internal mData is retrieved from the active layer in
                                    *          VectorLayerSet mLayerSet using VectorLayer::getData
                                    *          during setActiveLayer call.
                                    */

  SelectionSet    *mSelSet;        /*!< List of selected vector objects.
                                    *
                                    *@internal mSelSet is retrieved from mData.
                                    */

  // SelectionSet    mLastSelSet;
  bool            mInObject;
  bool            mCtrlDown;
  bool            mShiftDown;
  bool            mMovingNode;

  std::vector<SelectionSet::SelObj>  mOverNode;

  std::vector<SelectionSet::SelObj>  overSelNode( double x, double y );

  // HAS INTERSECT?
  /**
   * Answers the Question: do the lines described by p1-p2 and p3-p4 intersect?
   */
  bool hasIntersect( Node p1, Node p2, Node p3, Node p4 );


  // IS SELected LINE?
  /**
   * Answer to the question: Is the line represented by iline in the current
   * SelectionSet* mSelSet?
   *
   * @param iline  Value to match to SelObj.obj in SelectionSet* mSelSet
   *
   * @return if  found, index in SelectionSet* mSelSet for line
   *         if !found, -1
   */
  int  isSelLine( int iline );

  // IS SELected POINT?
  /**
   * Answer to the question: Is the point (inode) in this line (iline) selected
   * (in the current SelectionSet* mSelSet)?
   *
   * @param iline  Line to check for selected point.
   *               Value to match to SelObj.obj in SelectionSet* mSelSet.
   *               iline will be changed to either -1 (not found)
   *               or index in SelectionSet* mSelect where line is
   *               represented.
   * @param inode  Point to check for selection status.
   *               Value to match to a node in SelObj.nodes
   *               (for SelObj.obj in SelectionSet* mSelSet matching
   *               the given line, iline). inode will be changed to
   *               either -1 (not found) or the index in SelObj.nodes
   *               where representing the point.
   *
   * @return if  found, true
   *         if !found, false
   */
  bool isSelPoint( int &iline, int &inode );

  void init( );

  int _int_min( int v1, int v2 ) {
    return( v1 < v2 ) ? v1 : v2;
  }

  int _int_max( int v1, int v2 ) {
    return( v1 > v2 ) ? v1 : v2;
  }

public:
  VectorEditor( VectorLayerSet *, ViewSet * );
  virtual ~VectorEditor( );

  void addView( View *view );
  void removeView( View *view );
 /**
  * Clear selected vector objects on all layers
  */
  void clearAllLayerSelections () ;

public slots:
  void viewMousePress( QMouseEvent *me, View::MouseState &ms );
  void viewMouseMove( QMouseEvent *me, View::MouseState &ms );
  void viewMouseRelease( QMouseEvent *me, View::MouseState &ms );
  void viewKeyPress( QKeyEvent *ke );
  void viewKeyRelease( QKeyEvent *ke );

  // VIEW SELection CHANGED
  /**
   * Sets vector data objects as selected.
   * Determines which vector data objects (points and/or lines)
   * are inside the selection box (marquis, "dancing ants")
   * the user has drawn, and sets those objects as selected.
   * If the user has depressed the shift-key, the objects
   * are added to the current list of selected objects,
   * otherwise the list of selected objects is cleared
   * before adding the current selection.
   *
   * @param box    the selection box (marquis, "dancing ants") the user
   *               has drawn for the purpose of selecting vector objects
   *               on the screen to make them active for moving, deleting,
   *               etc.
   */
  void viewSelChanged( SelectionSet::SelBox &box );
  void breakLine( );
  void joinLine( );
  void contLine( );
  void delSel( );
  void activeLayerChanged( int );

signals:
  void repaint( bool );
  void selBoxChanged( Rect & );
  void selectionChanged() ;
};

#endif





