#ifndef __VIEW_H
#define __VIEW_H

#include <qwidget.h>
#include <qevent.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qapplication.h>

#include <string>

#include "histogramdatasource.h"
#include "histogramuser.h"
#include "image.h"
#include "shape.h"
#include "viewstate.h"
#include "vectorlayerset.h"

/**
 * This presents the user with a empty widget that draws
 * onto itself the image specified by the project and all
 * layers that are visible in the layer set.  This captures
 * user events and emits signals for each allowing
 * classes such as VectorEditor to handle user input.
 */
class View : public QWidget, public HistogramDataSource {
  Q_OBJECT
  public:
  typedef enum {
    NO_TOOL,
    PAN_HAND,
    ZOOMIN_BOX,
    ZOOMOUT_BOX,
    ZOOMIN_INC,
    ZOOMOUT_INC,
    LINE,
    GLACIER_ID,
    POINTER,
    SELECTION
  } Tool;

/**
 * Structure to hold mouse position information.
 */
  typedef struct {
    double      x;
    double      y;
  } MousePos;

/**
 * Structure to hold box information.
 */
  typedef struct {
    Rect   dim;
    bool   moving;
    Rect   origpos;
    int    xoff;
    int    yoff;
  } Box;

/**
 * Structure to hold mouse state information.
 */
  typedef struct {
    MousePos    mCurPos;
    MousePos    mStartPos;
    MousePos    mLastPos;
    bool        mLeftButDown;
    bool        mRightButDown;
    bool        mMidButDown;
  } MouseState;

  static Tool       mTool;
  static HistogramUser  mGlbHistUser;

protected:
  ViewState        mState;
  MouseState       mMouse;
  Rect             mVirDims;
  Rect             mDspDims;
  Rect             mLnkDims;
  Rect             mLinkLimits;
  Box              mLinkBox;
  bool             mChildLinked;
  bool             mParentLinked;
  Rect             mSelBox;
  Image           *mImage;
  VectorLayerSet  *mLyrSet;
  bool             mDspImg;
  bool             mDspVec;
  QPixmap*         mCurScreen;
  QImage*          mCurImg;
  bool             mFullRedraw;
  QTimer*          mTimer;
  bool             mCTableNew;
  bool             mPaning;

  ImageFormat::ScaleType mSclType;


public:
  View( QWidget *parent );
  View( QWidget *parent, Image* img, VectorLayerSet * );
  virtual ~View( );

  void init( );
  void setImage( Image *img );
  void setVectorLayerSet( VectorLayerSet *lyrset ) {
    mLyrSet = lyrset;
  }
  void setLinkChild( View *linkview );
  void setLinkParent( View *linkview );
  void setZoomAttr( ZoomAttr attr );
  ZoomAttr* getZoomAttr( ) const;
  void captureImage( );
  void calcDims( );
  void fullRedraw( );
  const Rect& getVirDims( ) const;
  void dspToVir( double &x, double &y );
  void virToDsp( double &x, double &y );
  void dspToVir( int &x, int &y );
  void virToDsp( int &x, int &y );
  Rect virDimToDsp( const Rect &dim );
  void initView( );
  void resetView( );

  void loadCaption( ) {
    char buf[16];
    double z_per = mState.mZoom.factor * 100.0;
    sprintf( buf, "%5.2f", z_per );

    setCaption( ( mState.mName + "   @" + buf ).c_str() );
  }

  void setViewName( std::string name ) {
    mState.mName = name;
    loadCaption();
  }

  ViewState& getViewState( ) {
    return mState;
  }

  void centerOn( double x, double y );

  void setEditable( bool b ) {
    mState.mEditable = b;
  }

  MouseState& getMouseState( ) {
    return mMouse;
  }

  void setRSM( ImageFormat::ScaleType st ) {
    mSclType = st; fullRedraw();
  }
  ImageFormat::ScaleType getRSM( ) {
    return mSclType;
  }

public:
  void paintEvent( QPaintEvent * );
  void mouseMoveEvent( QMouseEvent * );
  void mousePressEvent( QMouseEvent * );
  void mouseReleaseEvent( QMouseEvent * );
  void keyPressEvent( QKeyEvent * );
  void keyReleaseEvent( QKeyEvent * );
  void resizeEvent( QResizeEvent * );

public:
  virtual char* getHistName( ) const;
  virtual HistogramSet getHistSet( ) const;

public slots:
  void timerDone();
  void setLinkLimits( Rect );
  void setVirDims( Rect );
  void setLinkBoxDims( Rect );
  void selBoxChanged( Rect & );
  void ctableChanged( );

signals:
  void parentVirDimsChanged( Rect );
  void parentLinkBoxChanged( Rect );
  void childVirDimsChanged( Rect );
  void mousePosChanged( View::MousePos, View::MousePos );
  void viewMouseMove( QMouseEvent *, View::MouseState & );
  void viewMousePress( QMouseEvent *, View::MouseState & );
  void viewMouseRelease( QMouseEvent *, View::MouseState & );
  void viewKeyPress( QKeyEvent * );
  void viewKeyRelease( QKeyEvent * );
  // void viewPaintEvent( QPainter &, Rect & );
};

#endif



