#ifndef __XMLSERIALIZEABLE_H
#define __XMLSERIALIZEABLE_H

#include "windows_defs.h"

#include <qdom.h>
#include <qfile.h>
#include <qdatetime.h>
#include <string>
#include <cstdlib>
#include <fstream>
#include <sstream>


/**
 * A very used class within GLIMSView that provides
 * functions for an object to serialize and deserialize
 * itself easily and efficiently using the XML constructs
 * within QT.  A class that extends this must implement
 * the abastract methods that handle I/O.
 *
 * @history dls Added date and time functions
 * @history dls Added versioning functions
 */
class GV_EXPORT XMLSerializeable {

protected:
  std::string mID;
  std::string mBaseName;
  QDate mCurrentVersion;

  virtual bool writeTextTag(
                           QDomDocument &doc,
                           QDomElement  &elem,
                           std::string  tagName,
                           std::string  text,
                           std::string  id = "" );

  virtual bool writeIntTag(
                          QDomDocument &doc,
                          QDomElement  &elem,
                          std::string  tagName,
                          int          val,
                          std::string  id = "" ) {
    char buf[128];
    sprintf( buf, "%d", val );
    return writeTextTag( doc, elem, tagName, buf, id );
  }

  virtual bool writeDoubleTag(
                             QDomDocument &doc,
                             QDomElement  &elem,
                             std::string  tagName,
                             double       val,
                             std::string  id = "" ) {
    char buf[128];
    sprintf( buf, "%10.5f", val );
    return writeTextTag( doc, elem, tagName, buf, id );
  }

  virtual std::string readTextTag(
                                 QDomElement elem,
                                 std::string tagName = "",
                                 std::string id = "" );

  virtual int readIntTag(
                        QDomElement elem,
                        std::string tagName = "",
                        std::string id = "" ) {
    return atoi( readTextTag( elem, tagName, id ).c_str() );
  }

  virtual double readDoubleTag(
                              QDomElement elem,
                              std::string tagName = "",
                              std::string id = "" ) {
    return atof( readTextTag( elem, tagName, id ).c_str() );
  }

  virtual QDomElement createBaseTag( QDomDocument &doc,
                                     QDomElement &elem,
                                     std::string id,
                                     std::string tagName ) {
    QDomElement base = doc.createElement( tagName.c_str() );
    if ( id != "" )
      base.setAttribute( "id", id.c_str() );
    elem.appendChild( base );
    return base;
  }

  virtual QDomElement getElemByTagName( QDomElement &elem,
                                        std::string name );



  // functions for storing and retreiving ISO-8601 compliant date/time strings
  virtual std::string qDateToString (const QDate * aQDate) ;
  virtual bool stringToQDate (const std::string aStrDate, QDate * aQDate) ;

  virtual std::string qDateTimeToString (const QDateTime * aQDateTime) ;
  virtual bool stringToQDateTime (const std::string aStrDateTime, QDateTime * aQDateTime) ;


public:
  virtual ~XMLSerializeable( ) {
  }

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) = 0;
  virtual bool toXMLFile( std::string fname, std::string docname );

  virtual bool fromXML( QDomElement &elem ) = 0;
  virtual bool fromXMLFile( std::string fname, std::string docname );

  virtual std::string getID( ) {
    return mID;
  }
  virtual std::string baseName( ) {
    return mBaseName;
  }


  virtual void setCurrentVersion (std::string versionString) ;
  virtual void setCurrentVersion (QDate versionQDate) ;
  virtual QDate       getCurrentVersion    () ;
  virtual std::string getCurrentVersionStr () ;

};

#endif



